#
# Copyright 2010 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#

#
# code to handle conditions
#

# return an array of lines in the condition text box
sub get_condition_text {
  my $Whash = shift;

  my $buffer = $Whash->{'condText'}->get_buffer();
  my $text = $buffer->get_text($buffer->get_start_iter(),
			       $buffer->get_end_iter(),
			       FALSE);
  return(split($/, $text));
}

#######################
#                     #
# add a new condition #
#                     #
# extract text from   #
# comboEntry box and  #
# call add_text_to_   #
# conditions()        #
#                     #
#######################
sub add_to_conditions {
  my $Whash = shift;

  my $newcondition;
  my $x = $Whash->{'condComboEntry'}->get_active();

  if ($x == -1) {
    $newcondition =
      ($Whash->{'condComboEntry'}->child)->get_text();
    ($Whash->{'condComboEntry'}->child)->set_text("");
  } else {
    $newcondition = 
      $Whash->{'condComboEntry'}->get_active_text() . "\n";
    ($Whash->{'condComboEntry'}->child)->set_text("");
  }

  add_text_to_conditions ($Whash, $newcondition);
}

#############################################
# pass in text and add it to the conditions #
#############################################
sub add_text_to_conditions {
  ($Whash, $text) = @_;

  # skip blank entries
  if ($text eq "") { return };

  my $buffer = $Whash->{'condText'}->get_buffer();
  my $oldtext = $buffer->get_text($buffer->get_start_iter(),
				  $buffer->get_end_iter(),
				  FALSE);

  if ($Whash->{'cond_saveends_checkbox'}->get_active() &&
     $text !~ /bloodied/) {
    chomp $text;
    $text .= " (save ends) \n";
  }
  if ($text !~ /\n$/) {
    $text .= "\n";
  }
  $buffer->set_text($oldtext . $text);

  # search the buffer for "bloodied" and tag the text
  my $start = $buffer->get_start_iter();
  my ($matchstart, $matchend);
  ($matchstart, $matchend) = $start->
    forward_search("bloodied", GTK_TEXT_SEARCH_VISIBLE_ONLY, undef);
  while ($matchstart) {
    # apply the red background tag to the new text
    $buffer->apply_tag_by_name("red_bg",
			       $matchstart, $matchend);
    $start = $matchend;
  ($matchstart, $matchend) = $start->
    forward_search("bloodied", GTK_TEXT_SEARCH_VISIBLE_ONLY, undef);
  }

  # mark that file needs to be saved
  $Vars->{'modified'} = 1;
}

###############################################
# passed in text, figure out its position     #
# and then call delete_condition_at_position  #
# on it to delete it                          #
###############################################
sub delete_condition_text {

  my ($Whash, $text) = @_;
  my $buffer = $Whash->{'condText'}->get_buffer();

  # if buffer is empty, don't do anything
  if ($buffer->
      get_text($buffer->get_start_iter(),
	       $buffer->get_end_iter(),
	       FALSE) eq "") {
    return 0;
  }

  # look for text, when found find its position and delete it
  my $start = $buffer->get_start_iter();
  my ($matchstart, $matchend);
  ($matchstart, $matchend) = $start->
    forward_search($text, GTK_TEXT_SEARCH_VISIBLE_ONLY, undef);
  while ($matchstart) {
    my $pos = $matchstart->get_offset();
    delete_condition_at_position($Whash, $pos);
    my $start = $buffer->get_start_iter();
    ($matchstart, $matchend) = $start->
      forward_search($text, GTK_TEXT_SEARCH_VISIBLE_ONLY, undef);
  }
}



##################################################
# Code that actually deletes a line of text from #
# the buffer, given a starting position for it.  #
# It figures out the start and end of the line   #
# and then deletes that line.                    #
##################################################
sub delete_condition_at_position {
  my ($Whash, $pos) = @_;

  my $buffer = $Whash->{'condText'}->get_buffer();

  my $iter = $buffer->get_iter_at_offset($pos);

  # number of characters on the line
  my $chars = $iter->get_chars_in_line();

  # current line number of cursor
  my $line = $iter->get_line();
  #print "pos: $pos, char: $chars, line: $line \n";

  # set iterator to the beginning of the line
  my $start = $buffer->
    get_iter_at_line_offset($line, 0);

  # bump char to 1 if 0, so we avoid -1 below
  $chars++ if (!$chars);

  # set iterator to the end of the line
  my $end = $buffer->
    get_iter_at_line_offset($line, $chars-1);

  # move forward to go over the newline and
  # get to the start of the next line
  $end->forward_char();
  $buffer->delete($start, $end);

  # mark that file needs to be saved
  $Vars->{'modified'} = 1;

  return 0;
}

1;
