#
# Copyright 2010 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#

#
# Perl function to modify the windows
#

use strict;

############################
#                          #
# quit the program...maybe #
#                          #
############################
sub confirm_quit {

  my ($Whash, $Vars) = @_;
  my $unsaved = "";

  if ($Vars->{'modified'}) {
    $unsaved .= "\n(There are unsaved changes.)";
  }

  my $dialog = Gtk2::MessageDialog->new($Whash->{'window'},
					'destroy-with-parent',
					'question',
					'yes-no',
					"Quit Dice Minion?" . $unsaved);
  my $response = $dialog->run;
  if ($response eq 'yes') {
    Gtk2->main_quit;
  }
  $dialog->destroy();
}

#
#
#
sub show_about {

  my ($Whash, $Vars) = @_;
  my $aboutText = <<EOT;
$Vars->{'DMname'}
Copyright 2010 by Frank Adelstein.  All Rights Reserved.
Written by Frank Adelstein.
Last updated: 2010-08-30
Send comments, questions, suggestions to frank\@notfrank.com.
Information at: http://frank.notfrank.com/Programs/dm.html
EOT

  my $dialog = Gtk2::MessageDialog->new($Whash->{'window'},
					'destroy-with-parent',
					'info',
					'close',
					$aboutText);
  my $response = $dialog->run;
  if ($response eq 'close') {
    $dialog->destroy();
  }
}

#
#
#
sub show_help {

  my ($Whash, $Vars) = @_;

  # use the directory in which the program is located
  my $helpfile = dirname $0;
  $helpfile .= "/dm.help";
  my $helpText = "";
  if (!open (HF, "<", $helpfile)) {
    $helpText = "Help file [$helpfile] not found.  Sorry.";
  } else {
    while (<HF>) {
      $helpText .= $_;
    }
    close HF;
  }

  pop_up_dialog($Whash, 'Help', $helpText);


}


sub show_history {

  my $Whash = shift;
  my $Vars  = shift;
  my $history =  $Vars->{'undo'}->show || "No undo history.\n";
  pop_up_dialog($Whash, "Undo History", $history);
}


sub pop_up_dialog {
  my $Whash = shift;
  my $title = shift;
  my $text  = shift || "";	# set to blank if undef

  # create a dialog pop-up window
  my $dialog = Gtk2::Dialog->new($title, $Whash->{'window'},
				 'destroy-with-parent',
				 'close' => 'yes');

  # let it have a scroll bar, if needed
  my $scroll = Gtk2::ScrolledWindow->new(undef, undef);
  $scroll->set_shadow_type('etched-out');
  $scroll->set_policy('automatic', 'automatic');
  $scroll->set_size_request (400, 300);
  $dialog->get_content_area->add($scroll);
  
  # set the content to be the undo history
  my $label = Gtk2::Label->new;

  # set the text for the pop-up
  $label->set_markup($text);

  $label->set_alignment(0,0);
  $scroll->add_with_viewport($label);

  # only one button, so any response must be "ok" to get rid of it
  $dialog->signal_connect (response => sub {$_[0]->destroy });

  $dialog->show_all;
  return;

}


#
#
#
sub add_to_damage {
  my ($Whash, $Vars, $damage) = @_;
  my $tempHP = $Vars->{'tempHP'};
  my $curHP  = $Vars->{'curHP'};

  # if non-integer, just set damage to 0
  $damage = 0 if ($damage !~ /^-?\d+$/);

  if ($Vars->{'tempHP'} && $damage > 0) {
    # got temp HPs to burn and damage is not negative
    if ($Vars->{'tempHP'} > $damage) {
      # temp HPs absorb all the damage
      $Vars->{'tempHP'} -= $damage;
      $damage = 0;
    } else {
      $damage -= $Vars->{'tempHP'};
      $Vars->{'tempHP'} = 0;
    }
  }

  # add "bloodied" to condition if this damage crosses the line
  if (($Vars->{'curHP'} > $Vars->{'bloodied'}) &&
      ($Vars->{'curHP'} - $damage <= $Vars->{'bloodied'})) {
    add_text_to_conditions ($Whash, "bloodied");
  }


  # delete "bloodied condition if this "damage" heals over bloodied value
  if (($Vars->{'curHP'} <= $Vars->{'bloodied'}) &&
      ($Vars->{'curHP'} - $damage > $Vars->{'bloodied'})) {
    delete_condition_text ($Whash, "bloodied");
  }

  $Vars->{'curHP'} -= $damage;

  # save action to history
  $Vars->{'undo'}->do('damage', 
		      [\$Vars->{'curHP'}, \$Vars->{'tempHP'}], 
		      [$curHP, $tempHP], 
		      [$Vars->{'curHP'}, $Vars->{'tempHP'}]);

  update_UI ($Whash, $Vars);

  # mark that file needs to be saved
  $Vars->{'modified'} = 1;
}


#######################################
#                                     #
# Evaluate a dice expression          #
#                                     #
# input: $inputstring                 #
#                                     #
# output: $$ReturnSum, total of rolls #
#                                     #
#    $$ReturnRolls, string of the     #
#       roll values                   #
#                                     #
#    $$ReturnConst string of the      #
#       constant value                #
#                                     #
# inputstring format:                 #
# exp + ...                           #
#  where exp = ((x)d)y                #
#   where x and d are optional and d  #
#   is the letter 'd'                 #
#                                     #
# Ex:  4, d4, 2d4, d4+5, 2d4+3d8+1    #
#                                     #
#######################################

sub eval_dice_expression {
  my $inputstring  = shift;
  my $ReturnSum    = shift;
  my $ReturnRolls  = shift;
  my $ReturnConst  = shift;

  my $conststr = "";
  my $roll     = 0;
  my $sum      = 0;
  my $dicestr  = "";

  # split off each expression, separated by a '+' sign
  foreach my $exp (split /\+/,$inputstring) {
    $exp =~ m/((\d+)?([dD]))?(\d+)/;

    if (!defined($4)) {
      print "illegal expression [$exp] \n";
      return;
    }

    if (!defined($2) && !defined($3)) {
      # only a constant specified, just add it
      $sum += $4;
      $conststr .= "+$4 ";

    } elsif (!defined($2)) {
      # it's a dx expression, roll that die, record the value
      my $val = int(rand($4)) + 1;
      $sum += $val;
      $dicestr .= "d$4: $val, Total: $val\n";

      # if it's just a d20 report critical rolls
      if ($4 == 20) {
	if ($val == $4) {
	  $dicestr .= "Critical Success!\n";
	} elsif ($val == 1) {
	  $dicestr .= "Critical Failure!\n";
	}
      }

    } else {
      # should be full expression, iterate dice x times
      $roll = 0;
      $dicestr .= "d$4: ";

      for (my $count = 0; $count < $2; $count++) {
	my $val = int(rand($4)) + 1;
	$roll += $val;
	# record each roll
	$dicestr .= "$val, ";
      }
      # and record the total of the rolls for that die
      $dicestr .= "Total: $roll\n";
      $sum += $roll;
    }
  }

  $$ReturnSum   = $sum;
  $$ReturnRolls = $dicestr;
  $$ReturnConst = $conststr;

}



########################
#                      #
# roll dice            #
#                      #
# pop up a window with #
# the roll values      #
#                      #
########################

sub roll_dice {
  my $Whash   = shift;
  my $string  = shift;
  my $label   = shift;	# optional

  my $conststr = "";
  my $roll     = 0;
  my $sum      = 0;
  my $dicestr  = "";

  eval_dice_expression($string, \$sum, \$dicestr, \$conststr);
  my $text;
  if ($label) {
    $text = "Rolling $label\n<span><b>Total: $sum</b></span>\n$dicestr$conststr\n";
    $string = "$label $conststr";
  } else {
    $text = "<span><b>Roll: $sum</b></span>\n$dicestr$conststr\n";
  }

  my $dialog = Gtk2::MessageDialog->new_with_markup($Whash->{'window'},
						    'destroy-with-parent',
						    'other',
						    'close',
						    $text);

  $dialog->set_title("Rolled: $string");
  $dialog->signal_connect (response => sub { 
			     my $self = shift;
			     $self->destroy;
			   }
			  );
  $dialog->show_all;
}



##########################
#                        #
# roll attack and damage #
#                        #
# pop up a window with   #
# the attack and damage  #
# rolls in it            #
#                        #
##########################

sub roll_attack_and_damage {
  my $Whash   = shift;
  my $attack  = shift;
  my $damage  = shift;

  my $conststr1 = "";
  my $roll1     = 0;
  my $sum1      = 0;
  my $dicestr1  = "";

  my $conststr2 = "";
  my $roll2     = 0;
  my $sum2      = 0;
  my $dicestr2  = "";

  eval_dice_expression($attack, \$sum1, \$dicestr1, \$conststr1);
  my $text = "<span><b>Attack: $sum1</b></span>\n$dicestr1$conststr1\n($attack)\n\n";

  if ($damage ne "No") {
    eval_dice_expression($damage, \$sum2, \$dicestr2, \$conststr2);
    $text   .= "<span><b>Damage: $sum2</b></span>\n$dicestr2$conststr2\n($damage)\n";
  }


  my $dialog = Gtk2::MessageDialog->new_with_markup($Whash->{'window'},
					'destroy-with-parent',
					'info',
					'close',
					$text);

  $dialog->set_title("Attack and Damage");
  $dialog->signal_connect (response => sub { 
			     my $self = shift;
			     $self->destroy;
			   }
			  );
  $dialog->show_all;
}


#
#
#
sub update_defense {
  my $Whash = shift;
  my $Vars  = shift;
  my $name  = shift;

  if ($Whash->{'def' . $name . 'adj'}->get_value() != $Vars->{$name}) {
    # gray out defense if we've changed it
    $Whash->{'def' . $name . 'Label1'}->
      set_markup($Vars->{'defStr3'} . $Vars->{$name} . $Vars->{'defStr2'});
  } else {
    # set defense back to regular color
    $Whash->{'def' . $name . 'Label1'}->
      set_markup($Vars->{'defStr1'} . $Vars->{$name} . $Vars->{'defStr2'});
  }
}

#
#
#
sub update_abilities {
  my $Whash = shift;
  my $Vars  = shift;
  my $name  = shift;

  $Whash->{'abil' . $name . 'Label1'}->set_markup($Vars->{'defStr1'} .
						  $Vars->{$name} .
						  $Vars->{'defStr2'});

  my $sign = ($Vars->{$name . "Mod"} >= 0) ? "+" : " ";

  $Whash->{"abil" . $name . "Label2"}->set_markup($Vars->{'defStr1'} .
						  $sign .
						  $Vars->{$name . "Mod"} .
						  $Vars->{'defStr2'});
}

#
#
#
sub update_skills {
  my $Whash    = shift;
  my $Vars     = shift;
  my $name     = shift;
  my $position = shift;

  if ($position % 2) {
    $Whash->{"skill" . $name . "Label1"}->set_markup($Vars->{'skillbg1'} .
						     "+" . $Vars->{$name} .
						     $Vars->{'skillbg3'});
  } else {
    $Whash->{"skill" . $name . "Label1"}->set_markup($Vars->{'skillbg2'} .
						     "+" . $Vars->{$name} .
						     $Vars->{'skillbg3'});
  }
}

###########################
#                         #
###########################
sub update_UI {
  my $Whash = shift;
  my $Vars  = shift;

  # don't bother updating anything if we don't have a PC loaded
  if (!$Vars->{'level'}) {
    return;
  }

  # update the title
  my $title = $Vars->{'name'} . "- Level " . $Vars->{'level'} . " " .
    $Vars->{'race'} . " " . $Vars->{'class'} . " - " . $Vars->{'DMname'};

  $Whash->{'window'}->set_title($title);

  # update the HP label
  my $hpstr = $Vars->{'curHP'} . " / " . $Vars->{'maxHP'};

  # add temp HP if present
  if ($Vars->{'tempHP'}) {
    $hpstr .= " (+" . $Vars->{'tempHP'} . ")";
  }

  # if not bloodied
  if ($Vars->{'curHP'} > $Vars->{'bloodied'}) {
    $hpstr = $Vars->{'hpStr1'} . $hpstr;
  } else {
    $hpstr = $Vars->{'hpStr3'} .$hpstr;
  }

  $hpstr .= $Vars->{'hpStr2'};

  # set the HP label to the new value
  $Whash->{'hpLabel'}->set_markup($hpstr);


  $Whash->{'hpEntry'}->set_text("");


  # update healing surge value
  $Whash->{'hsvLabel'}->set_markup("Surge value: " . $Vars->{'surgeval'});

  # update healing surge remaining
  $Whash->{'hsrLabel'}->set_markup("Surges remaining:  " .
				   $Vars->{'remainingsurges'} .
				   " / " . $Vars->{'surges'});

  # update the 4 defenses
  # AC, FORT, REF, WILL
  update_defense($Whash, $Vars, "AC");
  update_defense($Whash, $Vars, "FORT");
  update_defense($Whash, $Vars, "REF");
  update_defense($Whash, $Vars, "WILL");

  # update 6 abilities
  # Str, Con, Dex, Int, Wis, Cha

  update_abilities($Whash, $Vars, "Str");
  update_abilities($Whash, $Vars, "Con");
  update_abilities($Whash, $Vars, "Dex");
  update_abilities($Whash, $Vars, "Int");
  update_abilities($Whash, $Vars, "Wis");
  update_abilities($Whash, $Vars, "Cha");

  # update 17 skills
  update_skills($Whash, $Vars, "Athletics", 0);
  update_skills($Whash, $Vars, "Acrobatics", 1);
  update_skills($Whash, $Vars, "Arcana", 2);
  update_skills($Whash, $Vars, "Bluff", 3);
  update_skills($Whash, $Vars, "Diplomacy", 4);
  update_skills($Whash, $Vars, "Dungeoneering", 5);
  update_skills($Whash, $Vars, "Endurance", 6);
  update_skills($Whash, $Vars, "Heal", 7);
  update_skills($Whash, $Vars, "History", 8);
  update_skills($Whash, $Vars, "Insight", 9);
  update_skills($Whash, $Vars, "Intimidate", 10);
  update_skills($Whash, $Vars, "Nature", 11);
  update_skills($Whash, $Vars, "Perception", 12);
  update_skills($Whash, $Vars, "Religion", 13);
  update_skills($Whash, $Vars, "Stealth", 14);
  update_skills($Whash, $Vars, "Streetwise", 15);
  update_skills($Whash, $Vars, "Thievery", 16);
  update_skills($Whash, $Vars, "Initiative", 17);

}
1;
