#
# Copyright 2010 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#

# healing functions: extended rest, short rest, and 
#         gain temp and permanent HPs with or without surges
#
#
# make the checkboxes enable or disable a dialog as they go on or off
# input: my widget type and the Widget Hash pointer
#
sub toggle_temp_hp { 
  my $this  = shift;
  my $Whash = shift;

  my $state = $this->get_active;
  if ($state) {
    # enable the temp HP and checkbox
    $Whash->{'dialogc4'}->set_sensitive(TRUE);
    $Whash->{'dialogs3'}->set_sensitive(TRUE);
  } else {
    # turn off 
    $Whash->{'dialogc4'}->set_sensitive(FALSE);
    $Whash->{'dialogs3'}->set_sensitive(FALSE);
  }
}

#
# make a pair of checkboxes enable or disable a dialog as they go on or off
# and behave like a radio button
# input: my checkbox (this), the other checkbox (that), and the Widget
# Hash pointer
#
sub heal_toggle_box { 
  my $this  = shift;
  my $that  = shift;
  my $Whash = shift;

  my $state = $Whash->{$this}->get_active;
  if ($state) {
    # enable the surge input
    $Whash->{'dialogl1'}->set_sensitive(TRUE);
    $Whash->{'dialoga1'}->set_sensitive(TRUE);
    if ($Whash->{$that}->get_active) {
      # turn off check 2, if we are now on and it was on
      $Whash->{$that}->set_active(FALSE);
    }
  } elsif (!$state && !$Whash->{$that}->get_active) {
    # turn off 
    $Whash->{'dialogl1'}->set_sensitive(FALSE);
    $Whash->{'dialoga1'}->set_sensitive(FALSE);
  }
}


######################################
#                                    #
# Heal, gain HPs or temp HPs         #
#                                    #
######################################
sub healing_rest {
  my ($Whash, $Vars) = @_;

  # build a dialog so the user can specify the type of healing to apply
  my $dialog = Gtk2::Dialog->new('Apply healing or Temporary HP', 
				 $Whash->{'window'},
				 'destroy-with-parent',
				'gtk-cancel' => 'cancel',
				'Accept'      => 'ok');
  my $vbox = Gtk2::VBox->new(FALSE, 6);
  $dialog->get_content_area()->add($vbox);

  my $frame1 = Gtk2::Frame->new('Healing');
  $vbox->pack_start($frame1, TRUE, TRUE, 0);
  $frame1->set_border_width(3);

  my $vbox1 = Gtk2::VBox->new(FALSE, 6);
  $frame1->add($vbox1);

  # "Spend healing surges" checkbox
  my $check1 = Gtk2::CheckButton->new_with_label("Spend healing surges");
  $check1->set_active(TRUE);
  $Whash->{'check1'} = $check1;
  $vbox1->pack_start($check1, TRUE, TRUE, 6);
  $check1->signal_connect(toggled => 
			  sub { heal_toggle_box("check1", "check2", $Whash)});

  # "Gain HP as if..." checkbox
  my $check2 = Gtk2::CheckButton->new_with_label("Gain HP as if healing surges were spent");
  $vbox1->pack_start($check2, TRUE, TRUE, 6);
  $Whash->{'check2'} = $check2;
  $check2->signal_connect(toggled => 
			  sub { heal_toggle_box("check2", "check1", $Whash)});

  my $hbox1 = Gtk2::HBox->new(FALSE, 6);
  $vbox1->pack_start($hbox1, TRUE, TRUE, 6);

  # "Number of healing surges" label and adjustment
  my $label1 = Gtk2::Label->new('Number of healing surges to use:');
  $hbox1->pack_start($label1, TRUE, TRUE, 6);
  $label1->set_alignment(0, 0);
  $Whash->{'dialogl1'} = $label1;

  # surges to spend adjuster/spinner
  my $adj1 = Gtk2::Adjustment->new(1, 1.0, $Vars->{'remainingsurges'}, 
				   1.0, 5.0, 0.0);
  my $spinner1 = Gtk2::SpinButton->new($adj1, 0, 0);
  $spinner1->set_wrap(TRUE);
  $hbox1->pack_start($spinner1, FALSE, TRUE, 6);
  $Whash->{'dialoga1'} = $spinner1;

  # make hitting Return in the spinner set its value and hit the OK button
  $spinner1->signal_connect(activate => 
			    sub { my $x = shift; 
				  $x->set_value($x->get_text);
				  $dialog->response('ok')
				}
			   );


  my $hbox2 = Gtk2::HBox->new(FALSE, 6);
  $vbox1->pack_start($hbox2, TRUE, TRUE, 6);

  # "Additional healing" label and adjustment
  my $label2 = Gtk2::Label->new('Additional healing:');
  $hbox2->pack_start($label2, TRUE, TRUE, 6);
  $label2->set_alignment(0, 0);

  # extra HPs adjuster/spinner
  my $adj2 = Gtk2::Adjustment->new(0.0, 0.0, 1000,
				   1.0, 5.0, 0.0);
  my $spinner2 = Gtk2::SpinButton->new($adj2, 0, 0);
  $spinner2->set_wrap(TRUE);
  $adj2->value(0);
  $hbox2->pack_start($spinner2, FALSE, TRUE, 6);
  $spinner2->grab_focus;

  # make hitting Return in the spinner set its value and hit the OK button
  $spinner2->signal_connect(activate => 
			  sub { my $x = shift; 
				$x->set_value($x->get_text);
				$dialog->response('ok')
			      }
			   );

  # frame for temp HPs
  my $frame2 = Gtk2::Frame->new('Temporary Hit Points');
  $vbox->pack_start($frame2, TRUE, TRUE, 6);
  $frame2->set_border_width(3);

  my $vbox2 = Gtk2::VBox->new(FALSE, 6);
  $frame2->add($vbox2);

  my $hbox3 = Gtk2::HBox->new(FALSE, 6);
  $vbox2->pack_start($hbox3, TRUE, TRUE, 6);

  # check box for gaining temp HPs
  my $check3 = Gtk2::CheckButton->new_with_label("Gain temporary hit points");
  $hbox3->pack_start($check3, TRUE, TRUE, 6);
  $check3->signal_connect(toggled => 
			  sub { toggle_temp_hp($check3, $Whash)});

  # temp HP adjuster/spinner
  my $adj3 = Gtk2::Adjustment->new(0.0, 1.0, 1000,
				   1.0, 5.0, 0.0);
  my $spinner3 = Gtk2::SpinButton->new($adj3, 0, 0);
  $spinner3->set_wrap(TRUE);
  $adj3->value(0);
  $spinner3->set_sensitive(FALSE);
  $Whash->{'dialogs3'} = $spinner3;
  $hbox3->pack_start($spinner3, FALSE, TRUE, 6);

  # make hitting Return in the spinner set its value and hit the OK button
  $spinner3->signal_connect(activate => 
			    sub { my $x = shift; 
				  $x->set_value($x->get_text);
				  $dialog->response('ok')
				}
			   );

  my $hbox4 = Gtk2::HBox->new(FALSE, 6);
  $vbox2->pack_start($hbox4, TRUE, TRUE, 6);

  # checkbox for stacking temp HPs
  my $check4 = Gtk2::CheckButton->new_with_label("Temp HP stacks with existing temp HP");
  $hbox4->pack_start($check4, TRUE, TRUE, 6);
  $check4->set_sensitive(FALSE);
  $Whash->{'dialogc4'} = $check4;

  # set control-Return to be the same as hitting "ok"
  my $accel2 = Gtk2::AccelGroup->new;
  $accel2->connect(Gtk2::Gdk->keyval_from_name('Return'), 
		   ['control-mask'], 
		   [],
		   sub {$dialog->response(GTK_RESPONSE_OK);}
		  );
  $dialog->add_accel_group($accel2);

  # pop up the dialog
  $dialog->show_all;
  my $response = $dialog->run;

  # if the user didn't hit cancel, then do the healing
  if ($response eq "ok") {
    # save current HP to check if we crossed the non-bloodied boundary
    my $curHP = $Vars->{'curHP'};
    my $curSurges = $Vars->{'remainingsurges'};
    my $curTempHP = $Vars->{'tempHP'};

    # first add additional healing (if any)
    if ($adj2->get_value ne 0) {
      $Vars->{'curHP'} += $adj2->get_value;
    }

    # if spend healing surges or gain as if spending a surge checkboxes 
    # are checked, add the appropriate number of surges to HPs

    if ($check1->get_active || $check2->get_active) {
      if ($adj1->get_value > $Vars->{'surges'}) {
	# only allow them to use how many surges are left
	# (should be guaranteed by the UI, but best to double check)
	$adj1->set_value($Vars->{'remainingsurges'});
      }
      $Vars->{'curHP'} += ($adj1->get_value) * $Vars->{'surgeval'};
      $Vars->{'curHP'} = $Vars->{'maxHP'} 
	if ($Vars->{'curHP'} > $Vars->{'maxHP'});
    }

    # if spend healing surges is active, then deduct the surges
    if ($check1->get_active) {
      $Vars->{'remainingsurges'} -= $adj1->get_value;
    }

    # finally, if temp HPs are selected, add them
    if ($check3->get_active) {
      if ($check4->get_active) {
	# user selected that temp HPs stack
	$Vars->{'tempHP'} += $adj3->get_value;
      } else {
	# temp HPs overwrite existing ones
	$Vars->{'tempHP'} = $adj3->get_value;
      }
    }

    # delete "bloodied condition if this healing takes us above bloodied value
    if (($Vars->{'curHP'} > $Vars->{'bloodied'}) &&
	($curHP <= $Vars->{'bloodied'})) {
      delete_condition_text ($Whash, "bloodied");
    }

    # add to the undo list
    $Vars->{'undo'}->do('healing', 
			[\$Vars->{'curHP'}, 
			 \$Vars->{'remainingsurges'}, 
			 \$Vars->{'tempHP'}], 
			[$curHP, $curSurges, $curTempHP], 
			[$Vars->{'curHP'}, 
			 $Vars->{'remainingsurges'}, 
			 $Vars->{'tempHP'}] );

    update_UI($Whash, $Vars);
    # mark that file needs to be saved
    $Vars->{'modified'} = 1;
  }
  $dialog->destroy();
}

######################################
#                                    #
# Take a short rest:                 #
# restore all used encounter powers  #
# and encounter magic items          #
#                                    #
######################################
sub short_rest {
  my ($Whash, $Vars) = @_;

  my $modified = 0;

  my $dialog = Gtk2::MessageDialog->new($Whash->{'window'},
					'destroy-with-parent',
					'question',
					'yes-no',
					"Take a short rest?");
  my $response = $dialog->run;
  if ($response eq 'yes') {

    # temporarily deactivate the "roll power" checkbox 
    # so we don't pop up dialog windows while setting used powers
    my $tempcheck = $Whash->{'powercheck'}->get_active;
    $Whash->{'powercheck'}->set_active(FALSE);

    # restore powers
    my $powers = $Vars->{'usedpowers'};
    foreach my $power (keys %$powers) {

      if ($powers->{$power}->{'usage'} =~ /Encounter/) {
	if (defined($powers->{$power}->{'count'})) {
	  # reset counter on Encounter (Special) powers
	  $powers->{$power}->{'count'} = 6;
	}
	toggle_power($Whash, $Vars, $powers->{$power});
	$modified++;
      }
    }


    # restore the "roll power" checkbox
    $Whash->{'powercheck'}->set_active($tempcheck);

    # restore items
    my $items = $Vars->{'useditems'};
    foreach my $item (keys %$items) {

      if ($items->{$item}->{'label'}->get_text =~ /^ \(E\) /) {
	# leading ' (E) ' means it was an encounter power on the item
	if (defined($items->{$item}->{'count'})) {
	  # Set count so that toggle_item() will make it roll over
	  $items->{$item}->{'count'} = 2;
	}
	toggle_item($Whash, $Vars, $items->{$item});
	$modified++;
      }
    }

  }

  # mark that file needs to be saved
  $Vars->{'modified'} = 1 if ($modified);

  $dialog->destroy();
}

######################################
#                                    #
# Take an extended rest:             #
# reset HPs to max and surges to MAX #
# also restore all powers (future)   #
#                                    #
######################################
sub extended_rest {
  my ($Whash, $Vars) = @_;

  my $dialog = Gtk2::MessageDialog->new($Whash->{'window'},
					'destroy-with-parent',
					'question',
					'yes-no',
					"Take an extended rest?");
  my $response = $dialog->run;
  if ($response eq 'yes') {
    $Vars->{'curHP'} = $Vars->{'maxHP'};
    $Vars->{'remainingsurges'} = $Vars->{'surges'};

    # reset all expended powers 
    my $powers = $Vars->{'usedpowers'};
    foreach my $power (keys %$powers) {
      if (defined($powers->{$power}->{'count'})) {
	# reset counter on Encounter (Special) powers
	$powers->{$power}->{'count'} = 6;
      }
	toggle_power($Whash, $Vars, $powers->{$power});
      $modified++;
    }

    # temporarily deactivate the "roll power" checkbox 
    # so we don't pop up dialog windows while resetting used powers
    my $tempcheck = $Whash->{'powercheck'}->get_active;
    $Whash->{'powercheck'}->set_active(FALSE);

    # reset all expended item powers 
    my $items = $Vars->{'useditems'};
    foreach my $item (keys %$items) {
      if (defined($items->{$item}->{'count'})) {
	# reset counter on Encounter (Special) powers
	$items->{$item}->{'count'} = 2;
      }
	toggle_item($Whash, $Vars, $items->{$item});
      $modified++;
    }


    # restore the "roll power" checkbox
    $Whash->{'powercheck'}->set_active($tempcheck);

    # reset action points
    $Whash->{'miscadj'}->set_value(1);

    # reset magic item uses, set appropriate to the tier 
    my $magicuses = 1;
    if ($Vars->{'level'}) {

      if ($Vars->{'level'} > 10) {
	$magicuses++;
      }
      if ($Vars->{'level'} > 20) {
	$magicuses++;
      }
    }
    
    $Whash->{'miscadj1'}->set_value($magicuses);

    # redraw things
    delete_condition_text ($Whash, "bloodied");
    update_UI($Whash, $Vars);

    # mark that file needs to be saved
    $Vars->{'modified'} = 1;
  }
  $dialog->destroy();

}


########################
#                      #
# adjust the number of #
# healing surges       #
#                      #
########################
sub adjust_surges {
  my ($Whash, $Vars) = @_;

  # build a dialog so the user can adjust the number of healing surges
  my $dialog = Gtk2::Dialog->new('Adjust number of healing surges remaining',
				 $Whash->{'window'},
				 'destroy-with-parent',
				 'gtk-cancel' => 'cancel',
				 'Accept'      => 'ok');

  my $vbox = Gtk2::VBox->new(FALSE, 6);
  $dialog->get_content_area()->add($vbox);

  my $hbox1 = Gtk2::HBox->new(FALSE, 6);
  $vbox->pack_start($hbox1, TRUE, TRUE, 6);

  # "Number of healing surges remaining" label and adjustment
  my $label1 = Gtk2::Label->new('Number of healing surges remaining:');
  $hbox1->pack_start($label1, TRUE, TRUE, 6);
  $label1->set_alignment(0, 0);
  $Whash->{'dialogl1'} = $label1;

  # surges remaining adjuster/spinner
  my $adj1 = Gtk2::Adjustment->new($Vars->{'remainingsurges'}, 1.0, 
				   $Vars->{'surges'}, 
				   1.0, 5.0, 0.0);
  my $spinner1 = Gtk2::SpinButton->new($adj1, 0, 0);
  $spinner1->set_wrap(TRUE);
  $hbox1->pack_start($spinner1, FALSE, TRUE, 6);
  $Whash->{'dialoga1'} = $spinner1;

  # make hitting Return in the spinner set its value and hit the OK button
  $spinner1->signal_connect(activate => 
			    sub { my $x = shift; 
				  $x->set_value($x->get_text);
				  $dialog->response('ok')
				}
			   );


  # set control-Return to be the same as hitting "ok"
  my $accel2 = Gtk2::AccelGroup->new;
  $accel2->connect(Gtk2::Gdk->keyval_from_name('Return'), 
		   ['control-mask'], 
		   [],
		   sub {$dialog->response(GTK_RESPONSE_OK);}
		  );
  $dialog->add_accel_group($accel2);

  # pop up the dialog
  $dialog->show_all;
  my $response = $dialog->run;

  # if the user didn't hit cancel, then do the healing
  if ($response eq "ok") {
    $Vars->{'remainingsurges'} = $adj1->get_value;
    update_UI($Whash, $Vars);
    # mark that file needs to be saved
    $Vars->{'modified'} = 1;
  }
  $dialog->destroy();
}

1;
