#
# Copyright 2010 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#

use strict;

##################################
# set up the frame and scrollbar #
# for the equipment window, will #
# be populated later             #
##################################
sub item_page_setup {
  my $Whash = shift;
  my $Vars  = shift;
  my $vbox4 = shift;
  my $label = shift;


  my $frame = Gtk2::Frame->new($label);
  $vbox4->pack_start($frame, FALSE, FALSE, 0);
  $frame->set_border_width(3);
  $Whash->{$label . 'Frame'} = $frame;

  # make a scrolled window
  my $scroll = Gtk2::ScrolledWindow->new(undef, undef);
  $scroll->set_shadow_type('etched-out');
  $scroll->set_policy('automatic', 'automatic');
  $scroll->set_size_request (100, 350);
  $frame->add($scroll);
  $Whash->{$label . 'Scroll'} = $scroll;

  my $vbox = Gtk2::VBox->new(FALSE, 6);
  $scroll->add_with_viewport($vbox);
  $Whash->{$label . 'Vbox'} = $vbox;

  # Magic item use tracker
  # for extra padding
  my $miscHbox2 = Gtk2::HBox->new(FALSE, 10);
  $vbox4->pack_end($miscHbox2, FALSE, TRUE, 6);
  $Whash->{"miscHbox2"} = $miscHbox2;

  # padding
  my $mischbox = Gtk2::HBox->new(FALSE, 6);
  $miscHbox2->pack_start($mischbox, FALSE, TRUE, 0);


  # magic item adjuster/spinner
  my $miscadj1 = Gtk2::Adjustment->new(1, 0.0,
				       10.0, 1.0, 5.0, 0.0);
  my $miscspinner1 = Gtk2::SpinButton->new($miscadj1, 0, 0);
  $miscspinner1->set_size_request (40, 27);
  $miscspinner1->set_wrap(TRUE);
  $miscHbox2->pack_start($miscspinner1, FALSE, FALSE, 0);

  $Whash->{"miscspinner1"} = $miscspinner1;
  $Whash->{"miscadj1"} = $miscadj1;

  # magic item use label
  my $miscAPLabel1 = Gtk2::Label->new("Magic Item Uses");
  $miscHbox2->pack_start($miscAPLabel1, FALSE, TRUE, 0);
#  $miscAPLabel1->set_size_request (50, 50);
  $miscAPLabel1->set_alignment(0, 0);
  $Whash->{"miscAPLabel1"} = $miscAPLabel1;


  return;

}

#
# Add a item 
#
sub add_item {

  my $Whash = shift;
  my $Vars = shift;
  my $item = shift;

  my $bcolor = 'orange';
  my $fcolor = 'black';
  my $maxlinesize = 50;

  # need an event box so we can track mouse clicks on the labels
  my $eventbox = Gtk2::EventBox->new;
  $eventbox->set_size_request (100, 20);
  $Whash->{'ItemsVbox'}->pack_start($eventbox, FALSE, FALSE, 0);

  # now make the label for the power
  my $label = Gtk2::Label->new(''); 

  $eventbox->modify_bg('normal', Gtk2::Gdk::Color->parse($bcolor));

  # pad out the line so the background color goes the length of it
  my $text = $item->{'name'};
  # drop the "(magic)" in the text if present
  $text =~ s/^(\(magic\))//;

  $label->set_markup("<span foreground=\"$fcolor\">" . 
		     $text . "</span>");
  $label->set_alignment(0, 0); 
  $label->set_size_request (100, 20);

  my $itemtooltip = Gtk2::Tooltips->new;
  $itemtooltip->set_tip($label, $text);
  $Whash->{"def" . $item->{'name'} . "tooltip"} = $itemtooltip;

  # store all this information in the item structure
  $item->{'label'} = $label;
  $item->{'eventbox'} = $eventbox;
  $item->{'fcolor'} = $fcolor;
  $item->{'bcolor'} = $bcolor;
  $item->{'text'} = $text;

  # build an associative array of items referenced by item name
  $Vars->{'itemsbyname'}->{$item->{'name'}} = $item;

  $eventbox->add($label);
  $eventbox->signal_connect(button_press_event => sub {
			      my $widget = shift;
			      my $event = shift;
			      if ($event->type eq '2button-press' ||
				  $event->type eq '3button-press') {
				# skip double/triple clicks
				return;
			      }
			      toggle_item($Whash, $Vars, $item);
			    }
			   );
} 


sub add_item_category {

  my $Whash = shift;
  my $Vars  = shift;
  my $name  = shift;
  my $aref  = shift;

  # add a separator/label 
  my $label = Gtk2::Label->new($name);
  $label->set_alignment(0, 0); 
  $label->set_size_request (100, 20);
  $Whash->{'ItemsVbox'}->pack_start($label, FALSE, FALSE, 6);

  # add entries for category
  foreach (@$aref) {
    add_item($Whash, $Vars, $_);
  }
  return;
}


#
# called when a new PC file is read in
#
sub add_items {

  my $Whash = shift;
  my $Vars = shift;

  my ($name, $usage, $type);
  my $aref;

  # go through the different arrays and add the items

  # add equipped items (if any exist)
  $aref = $Vars->{'items'}->{'Equipped'};
  add_item_category ($Whash, $Vars, "Equipped", $aref)
    if (scalar @$aref);

  # add weapon items (if any exist)
  $aref = $Vars->{'items'}->{'Weapons'};
  add_item_category ($Whash, $Vars, "Weapons", $aref)
    if (scalar @$aref);

  # add armor items (if any exist)
  $aref = $Vars->{'items'}->{'Armor'};
  add_item_category ($Whash, $Vars, "Armor", $aref)
    if (scalar @$aref);

  # add magic items (if any exist)
  $aref = $Vars->{'items'}->{'Magic'};
  add_item_category ($Whash, $Vars, "Magic", $aref)
    if (scalar @$aref);

  # add misc (non-magic) items (if any exist)
  $aref = $Vars->{'items'}->{'Misc'};
  add_item_category ($Whash, $Vars, "Misc", $aref)
    if (scalar @$aref);

  $Whash->{'notebookvbox3'}->show_all;
}


#
# toggle the power labels as used or unused
# note we can't discriminate between encounter and daily 
# so we'll just toggle between (E) and (D) in the name 
# to denote encounter and daily use
#
sub toggle_item {

  my ($Whash, $Vars, $item) = @_;

  my $prefix = "";

  # only magic items can be used
  if ($item->{'name'} =~ /\(magic\)/) {

    $Vars->{'modified'} = 1;

    # mark the power as used
    $Vars->{'useditems'}->{$item->{'name'}} = $item;
    # bump the use count
    $item->{'count'}++;

    if ($item->{'count'} == 1) {
      # set to encounter
      $prefix = " (E) ";
    } elsif ($item->{'count'} == 2) {
      # set to daily
      $prefix = " (D) ";
    } elsif ($item->{'count'} == 3) {
      # reset
      $item->{'count'} = 0;
      # unmark power
      delete $Vars->{'useditems'}->{$item->{'name'}};
    }

    if ($prefix) {
      # render with a prefix and striketrough
      $item->{'label'}->set_markup("<span foreground=\"$item->{'fcolor'}\" >" . $prefix . "<s>" . $item->{'text'} .  "</s></span>");

    } else {
      # render it with no strikethrough or prefix
      $item->{'label'}->set_markup("<span foreground=\"$item->{'fcolor'}\" >" . $item->{'text'} . "</span>");
    }
  }
}




1;
