#
# Copyright 2010 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#

use strict;

#
#
#
sub powers_page_setup {

  my $Whash = shift;
  my $Vars  = shift;
  my $vbox3 = shift;

  my ($name, $usage, $type);

  # create the 4 boxes for the different power types
  power_scrolled_setup($Whash, $Vars, $vbox3, 'Powers');

}


##################################
# set up the frame and scrollbar #
# for the powers page on the     #
# notebook; it's populated later #
##################################
sub power_scrolled_setup {

  my $Whash = shift;
  my $Vars  = shift;
  my $vbox3 = shift;
  my $name  = shift;
  my $label = shift;
  # 5th argument is optional
  $label = $name if (!defined($label));


  my $frame = Gtk2::Frame->new($label);
  $vbox3->pack_start($frame, FALSE, FALSE, 0);
  $frame->set_border_width(3);
  $Whash->{$name . 'Frame'} = $frame;

  # make a scrolled window
  my $scroll = Gtk2::ScrolledWindow->new(undef, undef);
  $scroll->set_shadow_type('etched-out');
  $scroll->set_policy('automatic', 'automatic');
  $scroll->set_size_request (100, 350);
  $frame->add($scroll);
  $Whash->{$name . 'Scroll'} = $scroll;

  my $vbox = Gtk2::VBox->new(FALSE, 6);
  $scroll->add_with_viewport($vbox);
  $Whash->{$name . 'Vbox'} = $vbox;

  # Action point tracker/roll power checkbox
  # for extra padding
  my $miscHbox1 = Gtk2::HBox->new(FALSE, 6);
  $vbox3->pack_end($miscHbox1, FALSE, TRUE, 0);
  $Whash->{"miscHbox1"} = $miscHbox1;

  # add the 'roll power' checkbox
  my $powercheck = Gtk2::CheckButton->new_with_label("Roll Power");
  $powercheck->set_active(TRUE);
  $Whash->{'powercheck'} = $powercheck;
  $miscHbox1->pack_start($powercheck, TRUE, TRUE, 0);

  # adjuster/spinner
  my $miscadj = Gtk2::Adjustment->new(1, 0.0,
				     10.0, 1.0, 5.0, 0.0);
  my $miscspinner = Gtk2::SpinButton->new($miscadj, 0, 0);
  $miscspinner->set_size_request (40, 27);
  $miscspinner->set_wrap(TRUE);
  $miscHbox1->pack_start($miscspinner, FALSE, FALSE, 0);

  $Whash->{"miscspinner"} = $miscspinner;
  $Whash->{"miscadj"} = $miscadj;

  # action point label
  my $miscAPLabel = Gtk2::Label->new("Action\nPoints");
  $miscHbox1->pack_start($miscAPLabel, FALSE, TRUE, 0);
  $miscAPLabel->set_size_request (50, 40);
  $miscAPLabel->set_alignment(0, 0);
  $Whash->{"miscAPLabel"} = $miscAPLabel;

  return;

}

#
# called when a new PC file is read in
#
sub add_powers {

  my $Whash = shift;
  my $Vars = shift;

  my ($name, $usage, $type);
  my (@atwill, @encounter, @daily, @utility);

  # iterate through the powers
  # separate them into 3 arrays for at-will/encounter/daily
  my $powers = $Vars->{'power'};
  foreach (@$powers) {
    # save the header information
    $name  = $_->{'name'};
    $usage = $_->{'usage'};
    $type  = $_->{'type'};

    my $wptr = $_->{'weapon'};
    if ($#$wptr == -1 && $usage !~ /At-Will/) {
      push @utility, $_;
    } elsif ($usage =~ /At-Will/) {
      push @atwill, $_;
    } elsif ($usage =~ /Encounter/) {
      push @encounter, $_;
    } elsif ($usage =~ /Daily/) {
      push @daily, $_;
    } else {
      print "Don't know [$usage] as a power usage for $name!\n";
    }
  }

  # add a separator/label 
  my $label = Gtk2::Label->new("At-Will Powers");
  $label->set_alignment(0, 0); 
  $label->set_size_request (100, 20);
  $Whash->{'PowersVbox'}->pack_start($label, FALSE, FALSE, 6);

  # add entries for the at-will powers
  foreach (@atwill) {
    add_power($Whash, $Vars, "atwill", $_);
  }

  # add a encounter powers label 
  $label = Gtk2::Label->new("Encounter Powers");
  $label->set_alignment(0, 0); 
  $label->set_size_request (100, 20);
  $Whash->{'PowersVbox'}->pack_start($label, FALSE, FALSE, 6);
  
  # add entries for the encounter powers
  foreach (@encounter) {
    add_power($Whash, $Vars, "encounter", $_);
  }

  # add a daily power label
  $label = Gtk2::Label->new("Daily Powers");
  $label->set_alignment(0, 0); 
  $label->set_size_request (100, 20);
  $Whash->{'PowersVbox'}->pack_start($label, FALSE, FALSE, 6);

  # add entries for the daily powers
  foreach (@daily) {
    add_power($Whash, $Vars, "daily", $_);
  }

  # add a utility power label
  $label = Gtk2::Label->new("Utility Powers");
  $label->set_alignment(0, 0); 
  $label->set_size_request (100, 20);
  $Whash->{'PowersVbox'}->pack_start($label, FALSE, FALSE, 6);

  # add entries for the utility powers
  foreach (@utility) {
    add_power($Whash, $Vars, "utility", $_);
  }

  $Whash->{'notebookvbox2'}->show_all;
}

#
# Add a power 
#
sub add_power {

  my $Whash = shift;
  my $Vars = shift;
  my $name = shift;
  my $power = shift;

  my $bcolor = 'yellow';
  my $fcolor = 'black';
  my $maxlinesize = 50;

  # set up foreground/background color names for the different powers
  if ($name eq 'atwill') {
    # green
    $bcolor = '#64E986';
  } elsif ($name eq 'encounter') {
    # magenta
    $bcolor = '#F778A1';
  } elsif ($name eq 'daily') {
    $bcolor = 'gray';
  } elsif ($name eq 'utility') {
    $bcolor = 'blue';
    $fcolor = 'white';
  }

  # need an event box so we can track mouse clicks on the labels
  my $eventbox = Gtk2::EventBox->new;
  $eventbox->set_size_request (100, 20);
  $Whash->{'PowersVbox'}->pack_start($eventbox, FALSE, FALSE, 0);

  # now make the label for the power
  my $label = Gtk2::Label->new(''); 

  $eventbox->modify_bg('normal', Gtk2::Gdk::Color->parse($bcolor));

  # pad out the line so the background color goes the length of it
  my $text = $power->{'name'};
  $label->set_markup("<span foreground=\"$fcolor\">" . 
		     $text . "</span>");
  $label->set_alignment(0, 0); 
  $label->set_size_request (100, 20);

  # set up a tool-tip that contains the attack information
  my $tooltip = Gtk2::Tooltips->new;
  my $tiptext = $_->{'type'} . " ($power->{'usage'}) \n";

  # add all the weapon attacks to the tool tip
  my $wref = $power->{'weapon'}; 
  foreach (@$wref) {
    $tiptext .=  "$_->{'name'}: $_->{'AttackBonus'} Vs. $_->{'Defense'}, $_->{'Damage'} damage\n";
  }

  # get rid of extra '\n' at the end
  chomp $tiptext;

  # set the tool tip
  $tooltip->set_tip($label, $tiptext);

  # store all this information in the power structure
  $power->{'label'} = $label;
  $power->{'eventbox'} = $eventbox;
  $power->{'tooltip'} = $tooltip;
  $power->{'fcolor'} = $fcolor;
  $power->{'bcolor'} = $bcolor;
  $power->{'text'} = $text;

  # build an associative array of powers referenced by power name
  $Vars->{'powersbyname'}->{$power->{'name'}} = $power;

  $eventbox->add($label);
  $eventbox->signal_connect(button_press_event => sub {
			      my $widget = shift;
			      my $event = shift;
			      if ($event->type eq '2button-press' ||
				  $event->type eq '3button-press') {
				# skip double/triple clicks
				return;
			      }

			      if ($power->{'name'} eq 'Second Wind') {
				# second wind is special

				if (!$Vars->{'usedpowers'}->
				    {$power->{'name'}}){
				 # power hasn't been used, make it heal 
				  healing_rest($Whash, $Vars);
				}

				# temporarily deactivate the "roll power" 
				my $tempcheck = $Whash->{'powercheck'}->get_active;
				$Whash->{'powercheck'}->set_active(FALSE);
				toggle_power($Whash, $Vars, $power);

				# restore the "roll power" checkbox
				$Whash->{'powercheck'}->set_active($tempcheck);

			      } else {
				toggle_power($Whash, $Vars, $power);
			      }
			    }
			   );
} 

#
#
#			  
sub roll_power {
  my $Whash = shift;
  my $Vars  = shift;
  my $power = shift;

  my $wref = $power->{'weapon'}; 
  if (!scalar @$wref) {
    # no weapons defined for this power
    return;
  }

  # build a dialog so the user can specify the type of healing to apply
  my $dialog = Gtk2::Dialog->new('Select Weapon for Attack', 
				 $Whash->{'window'},
				 'destroy-with-parent',
				 'gtk-cancel' => 'cancel',
				 'Accept'      => 'ok');
  my $vbox = Gtk2::VBox->new(FALSE, 6);
  $dialog->get_content_area()->add($vbox);

  my $frame1 = Gtk2::Frame->new('Weapons');
  $vbox->pack_start($frame1, TRUE, TRUE, 0);
  $frame1->set_border_width(3);

  my $vbox1 = Gtk2::VBox->new(FALSE, 6);
  $frame1->add($vbox1);

  my ($damage, $attack, $lastbox);
  # add all the weapon attacks to the tool tip
  foreach (@$wref) {

    # need an event box so we can track mouse clicks on the labels
    my $eventbox = Gtk2::EventBox->new;
    $eventbox->set_size_request (400, 20);
    $vbox1->pack_start($eventbox, FALSE, FALSE, 0);

    $eventbox->modify_bg('normal', Gtk2::Gdk::Color->parse('gray'));

    # add a label for each weapon
    my $label = Gtk2::Label->new;
    $label->set_alignment(0, 0); 
    $label->set_size_request (400, 20);
    #$Whash->{'PowersVbox'}->pack_start($label, FALSE, FALSE, 6);
    my $text .=  "$_->{'name'}: $_->{'AttackBonus'} Vs. $_->{'Defense'}, $_->{'Damage'} damage\n";
    $label->set_markup($text);
    my $tempd = $_->{'Damage'};
    my $tempa = $_->{'AttackBonus'};
    $eventbox->add($label);
#    $vbox1->pack_start($label, FALSE, TRUE, 0);
    $eventbox->signal_connect(button_press_event => sub {
				my $widget = shift;
				my $event = shift;
				if ($lastbox) {
				  # reset the last one back to gray
				  $lastbox->modify_bg('normal', Gtk2::Gdk::Color->parse('gray'));
				}
				$lastbox = $widget;
				# set the selected event box background color
				$lastbox->modify_bg('normal', Gtk2::Gdk::Color->parse('lightskyblue'));

				# save the attack and damage rolls 
				$damage = $tempd;
				$attack = $tempa;
			      }
			     );
  }

  # pop up the dialog
  $dialog->show_all;
  my $response = $dialog->run;
  $dialog->destroy;
  if ($response eq 'ok') {
    # roll the dice using the selected attack and damage values
    roll_attack_and_damage ($Whash, "d20+$attack", $damage);

  }

}

#
# toggle the power labels as used or unused
#
sub toggle_power {

  my ($Whash, $Vars, $power) = @_;

  if ($power->{'usage'} ne 'At-Will') {

    $Vars->{'modified'} = 1;

    if (!$Vars->{'usedpowers'}->{$power->{'name'}} &&
       $power->{'usage'} !~ /Special/) {
      # the power has not been used, so now make it have a strikethrough
      # with the <s> tag
      $power->{'label'}->set_markup("<span foreground=\"$power->{'fcolor'}\"><s><small>" . $power->{'text'} . "</small></s></span>");
      # mark the power as used
      $Vars->{'usedpowers'}->{$power->{'name'}} = $power;

      # roll the power if checkbox is checked
      if ($Whash->{'powercheck'}->get_active) {
	roll_power($Whash, $Vars, $power);
      }

    } elsif ($power->{'usage'} =~ /Special/) {

      # the power can be used multiple times so just bump up the counter
      my $count = "";
      $power->{'count'}++;
      # mark the power as used
      $Vars->{'usedpowers'}->{$power->{'name'}} = $power;
      if ($power->{'count'} > 5) {
	$power->{'count'} = "";
	# unmark power
	delete $Vars->{'usedpowers'}->{$power->{'name'}};
      }
      $count = " ($power->{'count'})" if ($power->{'count'});
      $power->{'label'}->set_markup("<span foreground=\"$power->{'fcolor'}\">" . $power->{'text'} . $count . "</span>");

      # roll the power if checkbox is checked
      if ($Whash->{'powercheck'}->get_active) {
	roll_power($Whash, $Vars, $power);
      }

    } else {
      # reset the power
      delete ($Vars->{'usedpowers'}->{$power->{'name'}});
      $power->{'label'}->set_markup("<span foreground=\"$power->{'fcolor'}\">" . $power->{'text'} . "</span>");

    }
  } else {
    # at-will powers just get used 
    # roll the power if checkbox is checked
    if ($Whash->{'powercheck'}->get_active) {
      roll_power($Whash, $Vars, $power);
    }
  }
}

1;
