#
# Copyright 2010 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#

use strict;

#
# Perl function to set up the windows
#

sub reset_defaults {

  my $Whash = shift;
  my $Vars  = shift;

  # clear out Vars array
  %{$Vars}  = ();

  # Versionm string
  $Vars->{'DMversion'} = "0.9";
  $Vars->{'DMname'} = "Dice Minion (Version $Vars->{'DMversion'})";

  # set some defaults
  $Vars->{'name'} = "";
  $Vars->{'level'} = "";
  $Vars->{'race'} = "";
  $Vars->{'class'} = "";
  $Vars->{'tempHP'} = 0;
  $Vars->{'maxHP'} = 0;
  $Vars->{'curHP'} = 0;
  $Vars->{'surges'} = 0;
  $Vars->{'surgeval'} = 0;
  $Vars->{'remainingsurges'} = 0;
  $Vars->{'bloodied'} = 1;
  $Vars->{'remainingsurges'} = 0;
  $Vars->{'defaultactionpoints'} = 1;
  
  $Vars->{'modified'} = 0;
  # create an empty hash to store what powers have been used
  $Vars->{'usedpowers'} = {};
  $Vars->{'powersbyname'} = {};

  # reset action points to one (if the UI component has been created)
  if (defined($Whash->{'miscadj'})) {
    $Whash->{'miscadj'}->set_value(1);
  }

  # reset magic item uses to one (if the UI component has been created)
  if (defined($Whash->{'miscadj1'})) {
    $Whash->{'miscadj1'}->set_value(1);
  }

  # second wind (if the checkbox has been created)
  if (defined($Whash->{'misccheck'})) {
    $Whash->{'misccheck'}->set_active(TRUE);
  }

  # reset ongoing damage
  if (defined ($Whash->{"hpOngoingEntry"})) {
    $Whash->{"hpOngoingEntry"}->set_text("");
  }

  # default conditions
  $Vars->{'Conditions'} = ['blinded', 'dazed', 'deafened', 'dominated',
			   'dying', 'helpless', 'immobilized', 
			   'insubstantial', 'marked', 'ongoing damage', 
			   'phasing', 'petrified', 'prone', 'restrained', 
			   'slowed', 'stunned', 'surprised', 'unconscious', 
			   'weakened'];


  # some format strings used for text 
  # bold and blue text for hit points
  $Vars->{'hpStr1'} = '<span foreground="DarkBlue" size="x-large"><b>  ';
  $Vars->{'hpStr2'} = '</b></span>';

  # bold and red text for hit points
  $Vars->{'hpStr3'} = '<span foreground="Red" size="x-large"><b>  ';

  # blue for defenses
  $Vars->{'defStr1'} = '<span foreground="DarkBlue" size="x-large">  ';
  $Vars->{'defStr2'} = '</span>';
  $Vars->{'defStr3'} = '<span foreground="Gray" size="x-large">  ';

  # two tone for alternating colors for skills
  $Vars->{'skillbg1'} = '<span foreground="DarkGoldenrod">';
  $Vars->{'skillbg2'} = '<span foreground="saddlebrown">';
  $Vars->{'skillbg3'} = '</span>';

  # for weapon background on power dialog pop-up
  $Vars->{'weaponbg1'} = '<span background="white">';
  $Vars->{'weaponbg2'} = '</span>';

  # if defense adjustments are visible, reset them
  foreach ('AC', 'FORT', 'REF', 'WILL') {
    my $w = $Whash->{'def' . $_ . 'button'};
    if (defined($w) && $w->visible) {
      # set the variable temporarily
      $Vars->{$_} = 1;
      $w->clicked;
      # get rid of the temporary value
      delete $Vars->{$_};
    }
  }

  # delete powers (label widgets)
  my $temp = $Whash->{'PowersVbox'};
  if (defined ($temp)) {
    my @children = $temp->get_children;
    foreach my $child (@children) {
      $temp->remove($child);
      $child->destroy;
    }
  }

  # delete items (label widgets)
  $temp = $Whash->{'ItemsVbox'};
  if (defined ($temp)) {
    my @children = $temp->get_children;
    foreach my $child (@children) {
      $temp->remove($child);
      $child->destroy;
    }
  }

  # delete conditions
  $temp = $Whash->{'condText'};
  if (defined($temp)) {
    $temp->get_buffer->set_text("");
  }

  # make an undo/history object
  $Vars->{'undo'} = History->new;

  return;
}

#
# Set up the drop down menus at the top of the window
#
sub make_menus {
  my ($Whash) = shift;
  my ($Vars) = shift;

  # menus bar
  my $menubar = Gtk2::MenuBar->new();
  $Whash->{'vbox0'}->pack_start($menubar, TRUE, TRUE, 0);

  # File menu
  my $ma = Gtk2::MenuItem->new("_File");
  $menubar->append($ma);
  my $m1 = Gtk2::Menu->new();

  # alt-F for File menu
  my $faccel = Gtk2::AccelGroup->new;
  $Whash->{'window'}->add_accel_group($faccel);


  # Open PC submenu
  my $m1a = Gtk2::MenuItem->new("_Open PC");
  $m1->append($m1a);

  # Open Monster submenu
  my $m1b = Gtk2::MenuItem->new("Open _Monster");
#  $m1->append($m1b);

  # Separator submenu
  my $m1c = Gtk2::SeparatorMenuItem->new();
  $m1->append($m1c);

  # Save PC submenu
  my $m1d = Gtk2::MenuItem->new("_Save PC");
  $m1->append($m1d);

  # Separator submenu
  my $m1e = Gtk2::SeparatorMenuItem->new();
  $m1->append($m1e);

  # Quit submenu
  my $m1f = Gtk2::MenuItem->new("_Quit");
  $m1->append($m1f);

  $ma->set_submenu($m1);


  # History menu
  my $mb = Gtk2::MenuItem->new("_History");
  $menubar->append($mb);
  my $m2 = Gtk2::Menu->new();
  $mb->set_submenu($m2);

  # Undo submenu
  my $m2a = Gtk2::MenuItem->new("_Undo");
  $m2->append($m2a);
  # Redo submenu
  my $m2b = Gtk2::MenuItem->new("_Redo");
  $m2->append($m2b);
  # History submenu
  my $m2c = Gtk2::MenuItem->new("_Show History");
  $m2->append($m2c);

  # Help menu
  my $mc = Gtk2::MenuItem->new("Help");
  $menubar->append($mc);
  my $m3 = Gtk2::Menu->new();
  $mc->set_submenu($m3);

  # Help submenu
  my $m3a = Gtk2::MenuItem->new("About");
  $m3->append($m3a);
  # About submenu
  my $m3b = Gtk2::MenuItem->new("Help");
  $m3->append($m3b);


  # bindings
  $m1a->signal_connect(activate => sub { 
			 read_file($Whash, $Vars);
		       });

  $m1b->signal_connect(activate => sub { print "File Monster\n"; });
  $m1d->signal_connect(activate => sub { 
			 write_dm_file($Whash, $Vars);
		       });

  $m1f->signal_connect(activate => sub { confirm_quit($Whash, $Vars); });
  $m2a->signal_connect(activate => sub { 
			 $Vars->{'undo'}->undo($Whash, $Vars); 
		       }
		      );
  $m2b->signal_connect(activate => sub { 
			 $Vars->{'undo'}->redo($Whash, $Vars); 
		       }
		      );
  $m2c->signal_connect(activate => sub { show_history($Whash, $Vars); });
  $m3a->signal_connect(activate => sub { show_about($Whash, $Vars); });
  $m3b->signal_connect(activate => sub { show_help($Whash, $Vars); });


  # keyboard shortcuts (accelerators)

  # control-O for open
  my $openaccel = Gtk2::AccelGroup->new;
  $Whash->{'window'}->add_accel_group($openaccel);

  $m1a->add_accelerator ('activate', $openaccel,
			 Gtk2::Gdk->keyval_from_name('O'), 
			 ['control-mask'], 
			 ['visible']);

  # control-Q for quit
  my $quitaccel = Gtk2::AccelGroup->new;
  $m1f->add_accelerator ('activate', $quitaccel,
			 Gtk2::Gdk->keyval_from_name('Q'), 
			 ['control-mask'], 
			 ['visible']);
  $Whash->{'window'}->add_accel_group($quitaccel);

}


###########################################
#                                         #
# main user interface creation and set up #
#                                         #
###########################################
sub Setup {
  my ($Whash, $Vars)  = @_;


  my $window = Gtk2::Window->new;
  $window->set_title ($Vars->{'DMname'});
  $window->signal_connect (destroy => sub { confirm_quit($Whash, $Vars); });
  $window->signal_connect (key_press_event =>
			   sub {
			     my ($w, $event) = @_;
			     return unless $event->keyval == 
			       $Gtk2::Gdk::Keysyms{Escape};
			     confirm_quit($Whash, $Vars);
			   });



  $window->set_border_width(3);
  $Whash->{"window"} = $window;

  # start off with a VBox
  my $vbox0 = Gtk2::VBox->new(FALSE, 6);
  $window->add($vbox0);
  $Whash->{"vbox0"} = $vbox0;

  make_menus($Whash, $Vars);

  # then an HBox
  my $hbox1 = Gtk2::HBox->new(FALSE, 6);
  $vbox0->pack_start($hbox1, TRUE, TRUE, 0);
  $Whash->{"hbox1"} = $hbox1;

  # the HBox has 3 VBoxes in it, here's the first one
  my $vbox1 = Gtk2::VBox->new(FALSE, 6);
  $hbox1->pack_start($vbox1, TRUE, FALSE, 0);
  $Whash->{"vbox1"} = $vbox1;


  ##############
  # hit points #
  ##############
  my $hpFrame = Gtk2::Frame->new('Hit Points');
  $vbox1->pack_start($hpFrame, FALSE, TRUE, 0);
  $hpFrame->set_border_width(3);
  $Whash->{"hpFrame"} = $hpFrame;

  # for extra padding
  my $hpHbox = Gtk2::HBox->new(FALSE, 6);
  $hpFrame->add($hpHbox);
  $hpHbox->set_border_width(3);
  $Whash->{"hpHbox"} = $hpHbox;

  my $hpVbox = Gtk2::VBox->new(FALSE, 6);
  $hpHbox->pack_start($hpVbox, TRUE, TRUE, 0);
  $Whash->{"hpVbox"} = $hpVbox;

  my $hpLabel = Gtk2::Label->new(""); 
  $hpVbox->pack_start($hpLabel, TRUE, TRUE, 0); 
  $hpLabel->set_markup($Vars->{'hpStr1'} . "0 / 0" . $Vars->{'hpStr2'});
  $hpLabel->set_alignment(0, 0); 
  $Whash->{"hpLabel"} = $hpLabel;


  my $hbox2 = Gtk2::HBox->new(FALSE, 0);
  $hpVbox->pack_start($hbox2, TRUE, TRUE, 0);
  $Whash->{"hbox2"} = $hbox2;

  my $hpEntry = Gtk2::Entry->new_with_max_length(6);
  $hpEntry->set_size_request (60, 30);
  $hbox2->pack_start($hpEntry, FALSE, FALSE, 0);
  $Whash->{"hpEntry"} = $hpEntry;

  # damage button
  my $damage_button = Gtk2::Button->new('Add _Damage');
#  $damage_button->set_size_request (60, 20);
  $hbox2->pack_end($damage_button, FALSE, TRUE, 0);
  $Whash->{"damage_button"} = $damage_button;

  # set the callbacks for damage
  $Whash->{'damage_button'}->signal_connect(clicked => sub {
					      my $damage = $Whash->
						{'hpEntry'}->get_text();
					      add_to_damage($Whash, $Vars, 
							    $damage);
					    }
					   );

  $Whash->{'hpEntry'}->signal_connect(activate => sub {
					my $damage = $Whash->
					  {'hpEntry'}->get_text();
					add_to_damage($Whash, $Vars, $damage);
				      }
				     );


  my $hbox3 = Gtk2::HBox->new(FALSE, 0);
  $hpVbox->pack_start($hbox3, TRUE, TRUE, 0);
  $Whash->{"hbox3"} = $hbox3;

  my $hpOngoingEntry = Gtk2::Entry->new_with_max_length(6);
  $hpOngoingEntry->set_size_request (60, 20);
  $hbox3->pack_start($hpOngoingEntry, FALSE, FALSE, 0);
  $Whash->{"hpOngoingEntry"} = $hpOngoingEntry;

  # damage button
  my $ongoing_damage_button = Gtk2::Button->new('Ongoing Da_mage');
  $hbox3->pack_end($ongoing_damage_button, FALSE, TRUE, 0);
  $ongoing_damage_button->set_size_request (130, 25);
  $Whash->{"ongoing_damage_button"} = $ongoing_damage_button;

  # set the callbacks for damage
  $ongoing_damage_button->
    signal_connect(clicked => sub {
		     my $damage = $Whash->{'hpOngoingEntry'}->get_text();
		     add_to_damage($Whash, $Vars, $damage);
		   }
		  );
  
  $Whash->{'hpOngoingEntry'}->
    signal_connect(activate => sub {
		     my $damage = $Whash->{'hpOngoingEntry'}->get_text();
		     add_to_damage($Whash, $Vars, $damage);
		   }
		  );
  


  ##############
  # conditions #
  ##############

  my $condFrame = Gtk2::Frame->new('Conditions');
  $vbox1->pack_start($condFrame, FALSE, TRUE, 0);
  $condFrame->set_border_width(3);
  $Whash->{"condFrame"} = $condFrame;

  # for extra padding
  my $condHbox = Gtk2::HBox->new(FALSE, 6);
  $condFrame->add($condHbox);
  $condHbox->set_border_width(3);
  $Whash->{"condHbox"} = $condHbox;

  my $condVbox = Gtk2::VBox->new(FALSE, 6);
  $condHbox->pack_start($condVbox, TRUE, TRUE, 6);
  $Whash->{"condVbox"} = $condVbox;

  # ComboBoxEntry
  my $condComboEntry = Gtk2::ComboBoxEntry->new_text;
  $condVbox->pack_start($condComboEntry, TRUE, TRUE, 0);
  $Whash->{"condComboEntry"} = $condComboEntry;

  # note: need to use the "child" method here because of
  # how gtk2 is implemented
  $condComboEntry->child()->
    signal_connect(activate => sub {
		     add_to_conditions($Whash);
		   }
		  );


  # add defaults to the menu
  my $aref =  $Vars->{'Conditions'};
  foreach (@$aref) {
    $condComboEntry->append_text ($_);
  }

  # scrolled window
  my $condScroll = Gtk2::ScrolledWindow->new(undef, undef);
  $condScroll->set_shadow_type('etched-out');
  $condScroll->set_policy('automatic', 'automatic');
#  $condScroll->set_size_request (100, 80);
  $condVbox->pack_start($condScroll, TRUE, TRUE, 0);

  # text widget
  my $condText = Gtk2::TextView->new();
  #$condVbox->pack_start($condText, TRUE, TRUE, 0);
  $condScroll->add($condText);
  $Whash->{"condText"} = $condText;
  my $mark = $condText->get_buffer()->
    create_mark("end", $condText->get_buffer()->get_end_iter(), FALSE);

  # make the text buffer scroll to the end when you add new text
  $condText->get_buffer()->
    signal_connect(insert_text => sub {
		     # scroll to end of buffer
		     $condText->scroll_to_mark ($mark, 0.0, TRUE, 0.0, 1.0);
		   }
		  );



  # make a tag to use when we want text with a red background
  my $redtag = $condText->get_buffer()->
    create_tag("red_bg", background => "red", foreground=> "white");
  $Whash->{"red_bg"} = $redtag;

  # make a tag to use when we want text with a gray background
  my $graytag = $condText->get_buffer()->
    create_tag("gray_bg", background => "gray", foreground=> "white");
  $Whash->{"gray_bg"} = $graytag;


  # hbox for the 2 buttons
  my $condbHbox = Gtk2::HBox->new(FALSE, 6);
  $condVbox->pack_start($condbHbox, TRUE, TRUE, 0);
  $Whash->{"condbHbox"} = $condbHbox;

  # healing button
  my $cond_rem_button = Gtk2::Button->new('_Remove');
  $condbHbox->pack_end($cond_rem_button, FALSE, FALSE, 0);
  $Whash->{"cond_rem_button"} = $cond_rem_button;

  $cond_rem_button->
    signal_connect('button-release-event' => sub {
		     my $textview = shift;
		     my $buffer = $Whash->{'condText'}->get_buffer();

		     # if buffer is empty, don't do anything
		     if ($buffer->
			 get_text($buffer->get_start_iter(),
				  $buffer->get_end_iter(),
				  FALSE) eq "") {
		       return 0;
		     }

		     # otherwise figure out the position where the user
		     # clicked and then call the delete_condition function
		     my $pos = $buffer->
		       get_property("cursor-position");
		     delete_condition_at_position($Whash, $pos);
		     # return false so other handlers will receive this signal
		     return 0;
		   }
		  );


  # add button
  my $cond_add_button = Gtk2::Button->new('_Add');
  $condbHbox->pack_end($cond_add_button, FALSE, FALSE, 0);
  $Whash->{"cond_add_button"} = $cond_add_button;

  $cond_add_button->
    signal_connect(clicked => sub {
		     add_to_conditions($Whash);
		   }
		  );

  # "save ends" checkbox
  my $cond_saveends_checkbox = Gtk2::CheckButton->
    new_with_mnemonic("(Save\n _Ends)");
  $condbHbox->pack_start($cond_saveends_checkbox, FALSE, FALSE, 0);
  $Whash->{"cond_saveends_checkbox"} = $cond_saveends_checkbox;


  ############
  # bindings #
  ############

  $condText->signal_connect('button-release-event' => sub {
			      my $textview = shift;
			      my $buffer = $textview->get_buffer();

			      # if buffer is empty, don't do anything
			      if ($buffer->
				  get_text($buffer->get_start_iter(),
					   $buffer->get_end_iter(),
					   FALSE) eq "") {
				return 0;
			      }

			      my $pos = $buffer->
				get_property("cursor-position");

			      my $i1 = $buffer->get_iter_at_offset($pos);
			      my $count = $i1->get_bytes_in_line;
			      $i1->forward_to_line_end;
			      $i1->backward_chars($count);

			      my $i2 = $buffer->get_iter_at_offset($pos);
			      $i2->forward_to_line_end;
			      $buffer->
				remove_all_tags ($buffer->get_start_iter,
						 $buffer->get_end_iter);
			      $buffer->apply_tag_by_name("gray_bg", $i1, $i2);

			      # make bloodied tags work again
			      my $start = $buffer->get_start_iter();
			      my ($matchstart, $matchend);
			      ($matchstart, $matchend) = $start->
				forward_search("bloodied", 'visible-only', undef);
			      while ($matchstart) {
				# apply the red background tag to the new text
				$buffer->apply_tag_by_name("red_bg",
							   $matchstart, $matchend);
				$start = $matchend;
				($matchstart, $matchend) = $start->
				  forward_search("bloodied", 'visible-only', undef);
			      }


			      # return false so other handlers will
			      # receive this signal
			      return 0;
			    });

  ##############
  # quick roll #
  ##############
  my $diceFrame = Gtk2::Frame->new('Quick Roll');
  $vbox1->pack_start($diceFrame, FALSE, FALSE, 6);
  $diceFrame->set_border_width(3);
  $Whash->{"diceFrame"} = $diceFrame;

  # for extra padding
  my $diceHbox = Gtk2::HBox->new(FALSE, 0);
  $diceFrame->add($diceHbox);
  $diceHbox->set_border_width(3);
  $Whash->{"diceHbox"} = $diceHbox;

  my $diceVbox = Gtk2::VBox->new(FALSE, 6);
  $diceHbox->pack_start($diceVbox, FALSE, TRUE, 0);
  $Whash->{"diceVbox"} = $diceVbox;

  # dice entry
  my $diceEntry = Gtk2::Entry->new_with_max_length(16);
  $diceEntry->set_size_request (80, 30);
  $diceHbox->pack_start($diceEntry, FALSE, FALSE, 6);
  $Whash->{"diceEntry"} = $diceEntry;

  # damage button
  my $dice_button = Gtk2::Button->new('_Roll');
  $diceHbox->pack_end($dice_button, FALSE, TRUE, 0);
  $Whash->{"dice_button"} = $dice_button;

  # has to be done after we've created the label so we can get to it
  $dice_button->signal_connect(clicked => sub {
				 roll_dice($Whash, $diceEntry->get_text);
			       }
			      );

  $diceEntry->signal_connect(activate => sub {
			       roll_dice($Whash, $diceEntry->get_text);
			     }
			    );




  # the top HBox has 3 VBoxes in it, here's the 2nd one
  my $vbox2 = Gtk2::VBox->new(FALSE, 6);
  $hbox1->pack_start($vbox2, TRUE, FALSE, 0);
  $Whash->{"vbox2"} = $vbox1;


  ############
  # defenses #
  ############
  my $defFrame = Gtk2::Frame->new('Defenses');
  $vbox2->pack_start($defFrame, FALSE, TRUE, 0);
  $defFrame->set_border_width(3);
  $Whash->{"defFrame"} = $defFrame;

  # for extra padding
  my $defHbox = Gtk2::HBox->new(FALSE, 0);
  $defFrame->add($defHbox);
  $defHbox->set_border_width(3);
  $Whash->{"defHbox"} = $defHbox;

  my $defVbox = Gtk2::VBox->new(FALSE, 0);
  $defHbox->pack_start($defVbox, FALSE, TRUE, 0);
  $Whash->{"defVbox"} = $defVbox;

  # table to hold label and value cells
  my $defTable = Gtk2::Table->new(4, 4, FALSE);
  $defVbox->pack_start($defTable, TRUE, TRUE, 0);
  $defTable->set_col_spacing(1, 5);
  $Whash->{"defTable"} = $defTable;

  # make the 4 defense stats displays
  make_defense ("AC", 0, $Vars, $Whash);
  make_defense ("FORT", 1, $Vars, $Whash);
  make_defense ("REF", 2, $Vars, $Whash);
  make_defense ("WILL", 3, $Vars, $Whash);

########################

  #############
  # abilities #
  #############
  my $abilFrame = Gtk2::Frame->new('Abilities');
  $vbox2->pack_start($abilFrame, FALSE, TRUE, 0);
  $abilFrame->set_border_width(3);
  $Whash->{"abilFrame"} = $abilFrame;

  # for extra padding
  my $abilHbox = Gtk2::HBox->new(FALSE, 10);
  $abilFrame->add($abilHbox);
  $abilHbox->set_border_width(3);
  $Whash->{"abilHbox"} = $abilHbox;

  my $abilVbox = Gtk2::VBox->new(FALSE, 6);
  $abilHbox->pack_start($abilVbox, FALSE, TRUE, 0);
  $Whash->{"abilVbox"} = $abilVbox;

  # table to hold label and value cells
  my $abilTable = Gtk2::Table->new(6, 3, FALSE);
  $abilVbox->pack_start($abilTable, TRUE, TRUE, 0);
  $abilTable->set_col_spacing(1, 20);
#  $abilTable->set_col_spacing(2, 20);
  $Whash->{"abilTable"} = $abilTable;

  # make the 6 ability stats displays
  make_ability ("Str", 0, $Vars, $Whash);
  make_ability ("Con", 1, $Vars, $Whash);
  make_ability ("Dex", 2, $Vars, $Whash);
  make_ability ("Int", 3, $Vars, $Whash);
  make_ability ("Wis", 4, $Vars, $Whash);
  make_ability ("Cha", 5, $Vars, $Whash);


  ##################
  # healing surges #
  ##################
  my $hsFrame = Gtk2::Frame->new('Healing Surges');
  $vbox2->pack_start($hsFrame, FALSE, TRUE, 0);
  $hsFrame->set_border_width(3);
  $Whash->{"hsFrame"} = $hpFrame;

  # for extra padding
  my $hsHbox = Gtk2::HBox->new(FALSE, 6);
  $hsFrame->add($hsHbox);
  $hsHbox->set_border_width(3);
  $Whash->{"hsHbox"} = $hpHbox;

  my $hsvbox = Gtk2::VBox->new(FALSE, 6);
  $hsHbox->pack_start($hsvbox, TRUE, TRUE, 6);
  $Whash->{"hsvbox"} = $hsvbox;

  my $hsvLabel = Gtk2::Label->new("");
  $hsvbox->pack_start($hsvLabel, TRUE, TRUE, 0);
  $hsvLabel->set_markup("Surge value: 0");
  $hsvLabel->set_alignment(0, 0);
  $Whash->{"hsvLabel"} = $hsvLabel;

  my $hshbox1 = Gtk2::HBox->new(FALSE, 0);
  $hsvbox->pack_start($hshbox1, TRUE, TRUE, 0);
  $Whash->{"hshbox1"} = $hshbox1;

  my $hsrLabel = Gtk2::Label->new("");
  $hshbox1->pack_start($hsrLabel, TRUE, TRUE, 0);
  $hsrLabel->set_markup("Surges remaining: 0 / 0");
  $hsrLabel->set_alignment(0, 0);
  $Whash->{"hsrLabel"} = $hsrLabel;

  # adjust (+/-) button
  my $hsmbutton = Gtk2::Button->new("");
  $hsmbutton->set_size_request(25,20);
  $Whash->{"hsmbutton"} = $hsmbutton;

  my $hstooltip = Gtk2::Tooltips->new;
  $hstooltip->set_tip($hsmbutton, "modify number of surges");
  $Whash->{"hstooltip"} = $hstooltip;
  $hshbox1->pack_start($hsmbutton, FALSE, FALSE, 10);

  $hsmbutton->signal_connect(clicked => sub {
			       adjust_surges($Whash, $Vars);
			     }
			    );
  

  # healing button
  my $healing_button = Gtk2::Button->new('_Healing/Temp HP');
  $hsvbox->pack_start($healing_button, FALSE, FALSE, 0);
  $Whash->{"healing_button"} = $healing_button;

  $healing_button->signal_connect(clicked => sub {
				    healing_rest($Whash, $Vars);
				  }
				 );
  # accelerator: control-H for healing/temp HP rest
  my $erestaccel2 = Gtk2::AccelGroup->new;
  $healing_button->add_accelerator ('clicked', $erestaccel2,
				    Gtk2::Gdk->keyval_from_name('H'), 
				    ['control-mask'], 
				    ['visible']);

  $Whash->{'window'}->add_accel_group($erestaccel2);


  # hbox for the 2 buttons
  my $hsrHbox = Gtk2::HBox->new(FALSE, 6);
  $hsvbox->pack_start($hsrHbox, FALSE, FALSE, 0);
  $Whash->{"hsrHbox"} = $hsrHbox;

  # healing button
  my $short_button = Gtk2::Button->new('_Short Rest');
  $hsrHbox->pack_start($short_button, FALSE, FALSE, 0);
  $Whash->{"short_button"} = $short_button;

  $short_button->signal_connect(clicked => sub {
				  short_rest($Whash, $Vars);
				}
			       );

  # accelerator: control-S for short rest
  my $erestaccel3 = Gtk2::AccelGroup->new;
  $short_button->add_accelerator ('clicked', $erestaccel3,
				  Gtk2::Gdk->keyval_from_name('S'), 
				  ['control-mask'], 
				  ['visible']);
  $Whash->{'window'}->add_accel_group($erestaccel3);


  # healing button
  my $extended_button = Gtk2::Button->new('E_xt\'d Rest');
  $hsrHbox->pack_start($extended_button, TRUE, TRUE, 0);
  $Whash->{"extended_button"} = $extended_button;

  $extended_button->signal_connect(clicked => sub {
				     extended_rest($Whash, $Vars);
				   }
				  );

  # accelerator: control-R for extended rest
  my $erestaccel1 = Gtk2::AccelGroup->new;
  $extended_button->add_accelerator ('clicked', $erestaccel2,
				     Gtk2::Gdk->keyval_from_name('X'), 
				     ['control-mask'], 
				     ['visible']);
  $Whash->{'window'}->add_accel_group($erestaccel1);




  #######
  # the top HBox has a notebook in it
  #######
  my $notebook = Gtk2::Notebook->new;
  $hbox1->pack_start($notebook, TRUE, TRUE, 0);
  $Whash->{'notebook'} = $notebook;
  $notebook->signal_connect('switch-page' => sub {
			      my $widget = shift;
			      my $page  = shift;
			      my $pagenum = shift;

			      if ($pagenum == 1 && 
				  !defined($Vars->{'powerpage'}) &&
				  defined($Vars->{'power'})) {
				# first time, so make the page refresh
				# we shouldn't have to do this, but the
				# scrollbar doesn't show up if we don't
				$Whash->{'notebookvbox2'}->hide_all;
				$Whash->{'notebookvbox2'}->show_all;
				$Vars->{'powerpage'} = 1;

			      } elsif ($pagenum == 2 &&
				       !defined ($Vars->{'itempage'}) &&
				       defined($Vars->{'items'})) {
				# first time, so make the page refresh
				# we shouldn't have to do this, but the
				# scrollbar doesn't show up if we don't
				$Whash->{'notebookvbox3'}->hide_all;
				$Whash->{'notebookvbox3'}->show_all;
				$Vars->{'itempage'} = 1;
			      }
			    }
			   );

  #######
  # Add a page to the notebook
  #######
  my $notebookvbox1 = Gtk2::VBox->new(FALSE, 6);
  $notebook->append_page($notebookvbox1, "Skills");
  $Whash->{"notebookvbox1"} = $notebookvbox1;

  # create the skills page
  skills_page_setup($Whash, $Vars, $notebookvbox1);


  #######
  # Add a second page to the notebook
  #######
  my $notebookvbox2 = Gtk2::VBox->new(FALSE, 6);
  $notebook->append_page($notebookvbox2, "Powers");
  $Whash->{"notebookvbox2"} = $notebookvbox2;

  # create the powers page
  powers_page_setup($Whash, $Vars, $notebookvbox2);


  #######
  # Add a third page to the notebook
  #######
  my $notebookvbox3 = Gtk2::VBox->new(FALSE, 6);
  $notebook->append_page($notebookvbox3, "Items");
  $Whash->{"notebookvbox3"} = $notebookvbox3;

  # create the items page
  item_page_setup($Whash, $Vars, $notebookvbox3, "Items");



  return $window;
}




#
# Set up all the stuff in the page that'll go on the Skills notebook tab
#
sub skills_page_setup {

  my $Whash = shift;
  my $Vars  = shift;
  my $vbox2  = shift;



  ##########
  # skills #
  ##########
  my $skillFrame = Gtk2::Frame->new('Skills');
  $vbox2->pack_start($skillFrame, FALSE, TRUE, 0);
  $skillFrame->set_border_width(3);
  $Whash->{"skillFrame"} = $skillFrame;

  # for extra padding
  my $skillHbox = Gtk2::HBox->new(FALSE, 10);
  $skillFrame->add($skillHbox);
  $skillHbox->set_border_width(3);
  $Whash->{"skillHbox"} = $skillHbox;

  my $skillVbox = Gtk2::VBox->new(FALSE, 6);
  $skillHbox->pack_start($skillVbox, FALSE, TRUE, 0);
  $Whash->{"skillVbox"} = $skillVbox;

  # table to hold label and value cells
  my $skillTable = Gtk2::Table->new(18, 2, FALSE);
  $skillVbox->pack_start($skillTable, TRUE, TRUE, 0);

  $Whash->{"skillTable"} = $skillTable;

  make_skill ("Acrobatics", 0, $Vars, $Whash);
  make_skill ("Arcana", 1, $Vars, $Whash);
  make_skill ("Athletics", 2, $Vars, $Whash);
  make_skill ("Bluff", 3, $Vars, $Whash);
  make_skill ("Diplomacy", 4, $Vars, $Whash);
  make_skill ("Dungeoneering", 5, $Vars, $Whash);
  make_skill ("Endurance", 6, $Vars, $Whash);
  make_skill ("Heal", 7, $Vars, $Whash);
  make_skill ("History", 8, $Vars, $Whash);
  make_skill ("Insight", 9, $Vars, $Whash);
  make_skill ("Intimidate", 10, $Vars, $Whash);
  make_skill ("Nature", 11, $Vars, $Whash);
  make_skill ("Perception", 12, $Vars, $Whash);
  make_skill ("Religion", 13, $Vars, $Whash);
  make_skill ("Stealth", 14, $Vars, $Whash);
  make_skill ("Streetwise", 15, $Vars, $Whash);
  make_skill ("Thievery", 16, $Vars, $Whash);
  make_skill ("Initiative", 17, $Vars, $Whash);

  return;
}

#
#
#
sub make_defense {

  my ($name, $position, $Vars, $Whash) = @_;

  my $table = $Whash->{'defTable'};

  # label
  my $defLabel = Gtk2::Label->new("");
  $table->attach($defLabel, 0, 1, $position, ($position+1), 
		 'shrink', 'shrink', 0, 0);
  $defLabel->set_size_request (70, 20);
  $defLabel->set_markup($Vars->{'defStr1'} . "$name: " . 
			$Vars->{'defStr2'});
  $defLabel->set_alignment(0, 0);
  $Whash->{"def" . $name . "Label"} = $defLabel;

  # value
  my $defLabel1 = Gtk2::Label->new("");
  $table->attach($defLabel1, 1, 2, $position, ($position+1),
		'shrink', 'shrink', 0, 0);
  $defLabel1->set_markup($Vars->{'defStr1'} . "0" .
			 $Vars->{'defStr2'});
  $defLabel1->set_alignment(0, 0);
  $Whash->{"def" . $name . "Label1"} = $defLabel1;

  # adjuster/spinner
  my $defadj = Gtk2::Adjustment->new(0, 1.0,
				     40.0, 1.0, 5.0, 0.0);
  my $defspinner = Gtk2::SpinButton->new($defadj, 0, 0);
  $defspinner->set_size_request (40, 30);
  $defspinner->set_wrap(TRUE);
#  $table->attach_defaults($defspinner, 2, 3, $position, ($position+1));
  $Whash->{"def" . $name . "spinner"} = $defspinner;
  $Whash->{"def" . $name . "adj"} = $defadj;

  # reset button
  my $defbutton = Gtk2::Button->new();
  my $tempdir = dirname $0;
  my $img = Gtk2::Image->new_from_file($tempdir . '/reset.png');
  $defbutton->set_image($img);
  $Whash->{"def" . $name . "button"} = $defbutton;
  my $deftooltip = Gtk2::Tooltips->new;
  $deftooltip->set_tip($defbutton, "reset to default value");
#  $table->attach_defaults($defbutton, 3, 4, $position, ($position+1));
  $Whash->{"def" . $name . "tooltip"} = $deftooltip;


  # placeholder/spacer label
  my $defLabel2 = Gtk2::Label->new("");
  $defLabel2->set_markup('  ');
  $defLabel2->set_size_request (40, 20);
  $table->attach($defLabel2, 2, 3, $position, ($position+1),
		 'shrink', 'shrink', 0, 0);
  $Whash->{"def" . $name . "Label2"} = $defLabel2;


  # adjust (+/-) button
  my $defadjbutton = Gtk2::Button->new("");
  $Whash->{"def" . $name . "adjbutton"} = $defadjbutton;
  my $defadjtooltip = Gtk2::Tooltips->new;
  $defadjtooltip->set_tip($defadjbutton, "modify defense value");
  $table->attach($defadjbutton, 3, 4, $position, ($position+1),
		 'shrink', 'shrink', 0, 0);
  $Whash->{"def" . $name . "adjtooltip"} = $defspinner;

  $defadj->signal_connect('value-changed' => sub {
			    my $tempname = 0;
			    $tempname = $Vars->{$name} 
			      if (defined($Vars->{$name}));
			    if ($defadj->get_value() != $tempname) {
			      # gray out defense if we've changed it
			      $defLabel1->set_markup($Vars->{'defStr3'} . 
						     $tempname .
						     $Vars->{'defStr2'});
			    } else {
			      # set defense back to regular color
			      $defLabel1->set_markup($Vars->{'defStr1'} . 
						     $tempname .
						     $Vars->{'defStr2'});
			    }
			    $Vars->{'modified'} = 1;
			  }
			 );

  $defbutton->signal_connect(clicked => sub {
			       my $w = shift;

			       # Only make changes if we've got a parent
			       # which means we're in the table.  Note: 
			       # reset_defaults() can call us from any state
			       if ($w->parent) {
				 $table->remove($defbutton);
				 $table->remove($defspinner);
				 $table->attach($defadjbutton, 3, 4,
						$position, $position+1,
						'shrink', 'shrink', 0, 0);
				 $table->attach($defLabel2, 2, 3, 
						$position, $position+1,
						'shrink', 'shrink', 0, 0);
				 $table->show_all;
			       }
			       
			       my $tempname = 0;
			       $tempname = $Vars->{$name} 
				 if (defined($Vars->{$name}));

			       if ($defadj->get_value() != $tempname) {
				 # reset adjustment widget to default if needed
				 $defadj->set_value($tempname);
				 $Vars->{'modified'} = 1;
			       }
			     }
			    );
  


  $defadjbutton->signal_connect(clicked => sub {
				  $table->remove($defadjbutton);
				  $table->remove($defLabel2);
				  $table->attach($defspinner, 2, 3, 
						 $position, $position+1,
						 'shrink', 'shrink', 0, 0);
				  $table->attach($defbutton, 3, 4,
						 $position, $position+1,
						 'shrink', 'shrink', 0, 0);
				  $table->show_all;
				}
				);
  
}

#
#
#
sub make_ability {

  my ($name, $position, $Vars, $Whash) = @_;

  my $table = $Whash->{'abilTable'};

  # label
  my $defLabel = Gtk2::Label->new("");
  $table->attach_defaults($defLabel, 0, 1, $position, ($position+1));
  $defLabel->set_size_request (70, 20);
  $defLabel->set_markup($Vars->{'defStr1'} . "$name: " .
			$Vars->{'defStr2'});
  $defLabel->set_alignment(0, 0);
  $Whash->{"abil" . $name . "Label"} = $defLabel;

  # value
  my $defLabel1 = Gtk2::Label->new("");
  $table->attach_defaults($defLabel1, 1, 2, $position, ($position+1));
  $defLabel1->set_markup($Vars->{'defStr1'} . "0" .
			 $Vars->{'defStr2'});
  $defLabel1->set_alignment(0, 0);
  $Whash->{"abil" . $name . "Label1"} = $defLabel1;

  # modifier
  my $defLabel2 = Gtk2::Label->new("");
  $table->attach_defaults($defLabel2, 2, 3, $position, ($position+1));
  # just use a plus when initially creating the window
  my $sign = "+";
  $defLabel2->set_markup($Vars->{'defStr1'} . $sign .
			 "0" .
			 $Vars->{'defStr2'});
  $defLabel2->set_alignment(0, 0);
  $Whash->{"abil" . $name . "Label2"} = $defLabel2;

  # reset button
#  my $defbutton = Gtk2::Button->new();
#  my $img = Gtk2::Image->new_from_file("./reset.png");
#  $defbutton->set_image($img);
#  my $deftooltip = Gtk2::Tooltips->new;
#  $deftooltip->set_tip($defbutton, "roll");
#  $table->attach_defaults($defbutton, 3, 4, $position, ($position+1));
#  $Whash->{"def" . $name . "tooltip"} = $defbutton;

}

#
#
#
sub make_skill {

  my ($name, $position, $Vars, $Whash) = @_;
  my $table = $Whash->{'skillTable'};

  # need an event box so we can track mouse clicks on the labels
  my $eventbox = Gtk2::EventBox->new;
  $eventbox->set_size_request (130, 20);
  $table->attach_defaults($eventbox, 0, 1, $position, ($position+1));
#  $Whash->{'ItemsVbox'}->pack_start($eventbox, FALSE, FALSE, 0);

  # name label
  my $skillLabel = Gtk2::Label->new("");
#  $table->attach_defaults($skillLabel, 0, 1, $position, ($position+1));
  $eventbox->add($skillLabel);
  $skillLabel->set_size_request (130, 20);
  if ($position % 2) {
    $skillLabel->set_markup($Vars->{'skillbg1'} .
			    "$name: " . $Vars->{'skillbg3'});
  } else {
    $skillLabel->set_markup($Vars->{'skillbg2'} .
			    "$name: " . $Vars->{'skillbg3'});
  }
  $skillLabel->set_alignment(0, 0);
  $Whash->{"skill" . $name . "Label"} = $skillLabel;

  my $skilltooltip = Gtk2::Tooltips->new;
  $skilltooltip->set_tip($skillLabel, "click to roll");
  $Whash->{"skill" . $name . "tooltip"} = $skilltooltip;

  $eventbox->signal_connect(button_press_event => sub {
			      my $widget = shift;
			      my $event = shift;
			      my $str = "d20";
			      $str .= "+$Vars->{$name}"
				if (defined ($Vars->{$name}));
			      roll_dice($Whash, $str, $name);
			      return;
			    }
			   );



  # make value
  my $skillLabel1 = Gtk2::Label->new("");
  $table->attach_defaults($skillLabel1, 1, 2, $position, ($position+1));
  $skillLabel1->set_size_request (50, 20);
  if ($position % 2) {
    $skillLabel1->set_markup($Vars->{'skillbg1'} . "+0" . $Vars->{'skillbg3'});
  } else {
    $skillLabel1->set_markup($Vars->{'skillbg2'} . "+0" . $Vars->{'skillbg3'});
  }
  $skillLabel1->set_alignment(1, 0);
  $Whash->{"skill" . $name . "Label1"} = $skillLabel1;
}

1;
