#
# Copyright 2010 Frank Adelstein.  All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this
# software is hereby granted without fee, provided that
# the copyright notice and permission notice are not removed.
#

require XML::Simple;
use XML::Simple qw(:strict);
use File::Basename;

use Data::Dumper;
use strict;

##########################
#                        #
# read in a PC data file #
#                        #
##########################
sub read_file {
  my ($Whash, $Vars) = @_;
  my $xml;

  my $f = Gtk2::FileChooserDialog->new("Read PC file", $Whash->{'window'},
				       "open",
				       'gtk-cancel' => 'cancel',
				       'Open' => 'ok');
  my $dmFiles = Gtk2::FileFilter->new;
  $dmFiles->set_name('Dice Minion Files');
  $dmFiles->add_pattern('*.dm');

  my $d4eFiles = Gtk2::FileFilter->new;
  $d4eFiles->set_name('D&D4e Files');
  $d4eFiles->add_pattern('*.dnd4e');

  my $allFiles = Gtk2::FileFilter->new;
  $allFiles->set_name('All Files');
  $allFiles->add_pattern('*');

  $f->add_filter($dmFiles);
  $f->add_filter($d4eFiles);
  $f->add_filter($allFiles);

  my $res = $f->run;
  my $file = $f->get_filename();
  $f->destroy;

  # create and pop up a file browser dialog window
  my $dialog = Gtk2::Dialog->new('Loading PC data', 
				 $Whash->{'window'},
				 'destroy-with-parent');
  my $label = Gtk2::Label->new('Loading PC data, \n\n\nplease wait.');
  $dialog->get_content_area()->add($label);
  $dialog->show_all;

  # if the user clicked the OK button, then read in the XML file 
  # and update the UI for the new values
  if ($res eq 'ok') {

    # reset the variables
    reset_defaults($Whash, $Vars);


    $Vars->{'PCFile'} = $file;
    # get the file suffix
    my ($name, $path, $suffix) = fileparse($file, qr/\.[^.]*/);
    #print "name = $name, path = $path, suffix = $suffix\n";

    my @usedpowers;
    my @useditems;
    my ($ACadj, $FORTadj, $REFadj, $WILLadj);

    if ($suffix eq '.dnd4e') {
      # it's a Character Builder PC file
      if (!read_xml($file, $xml, $Vars)) {
	$Vars->{'curHP'} = $Vars->{'maxHP'};
	$Vars->{'remainingsurges'} = $Vars->{'surges'};

	# add the powers to the UI
	add_powers($Whash, $Vars);

	# add the items to the UI
	add_items($Whash, $Vars);

	update_UI($Whash, $Vars);

	# set the values so the defenses won't be grayed out
	$Whash->{'defACadj'}->set_value($Vars->{'AC'});
	$Whash->{'defFORTadj'}->set_value($Vars->{'FORT'});
	$Whash->{'defREFadj'}->set_value($Vars->{'REF'});
	$Whash->{'defWILLadj'}->set_value($Vars->{'WILL'});

	$dialog->destroy;
	return;

      } else {
	print "Can't parse file\n";
      } 
    } elsif ($suffix eq '.dm') {
      # read in the file and then read in the character builder file
      my $fh;
      if (!open ($fh, "<", $file)) {
	# pop up a warning message
	print "Can't open file $file for read!!\n";
	return;
      }

      # read in the fields
      while (<$fh>) {
	#print $fh "ID: DM PC file\n";
	#$Vars->{'version'}\n";
	if (/^PCFile: (.*)$/) {
	  $Vars->{'PCFile'} = $1;
	} elsif (/^curHP: (.*)$/) {
	  $Vars->{'curHP'} = $1;
	} elsif (/^remainingsurges: (.*)$/) {
	  $Vars->{'remainingsurges'} = $1;
	} elsif (/^condition: (.*)$/) {
	  add_text_to_conditions($Whash, $1);
	} elsif (/^ACadjust: (.*)$/) {
	  $ACadj = $1;
	} elsif (/^FORTadjust: (.*)$/) {
	  $FORTadj = $1;
	} elsif (/^REFadjust: (.*)$/) {
	  $REFadj = $1;
	} elsif (/^WILLadjust: (.*)$/) {
	  $WILLadj = $1;
	} elsif (/^actionpoints: (.*)$/) {
	  $Whash->{'miscadj'}->set_value($1);
	} elsif (/^magicitemuses: (.*)$/) {
	  $Whash->{'miscadj1'}->set_value($1);
	} elsif (/^ongoing: (.*)$/) {
	  $Whash->{"hpOngoingEntry"}->set_text($1);
	} elsif (/^usedpower: (.*)$/) {
	  # save used power to deal with later (after reading them in)
	  push @usedpowers, $1;
	} elsif (/^useditem: (.*)$/) {
	  # save used item to deal with later (after reading them in)
	  push @useditems, $1;
	}
      }
      # close the file
      close $fh;
    }


    if (! -r $Vars->{'PCFile'}) {
      # if the file isn't readable (maybe it moved?)
      # give the user a chance to specify the new location

      $f = Gtk2::FileChooserDialog->new("PC file unavailable, pick again", 
					$Whash->{'window'},
					"open",
					'gtk-cancel' => 'cancel',
					'Open' => 'ok');
      $f->add_filter($dmFiles);
      $f->add_filter($allFiles);

      $res = $f->run;
      $file = $f->get_filename();
      $f->destroy;

      # if the user clicked the OK button, then read in the XML file 
      # and update the UI for the new values
      if ($res eq 'ok') {

	$Vars->{'PCFile'} = $file;
	# get the file suffix
	my ($name, $path, $suffix) = fileparse($file, qr/\.[^.]*/);
	print "name = $name, path = $path, suffix = $suffix\n";

	if ($suffix ne '.dnd4e') {
	  # bail out, they picked the wrong file
	  return;
	}
      } else {
	# bail out, they selected "cancel"
	return;
      }
      # otherwise, they selected a new, maybe valid, file
    }

    # now read in the character builder file
    if (!read_xml($Vars->{'PCFile'}, $xml, $Vars)) {

      # add the powers to the UI
      add_powers($Whash, $Vars);

      # temporarily deactivate the "roll power" checkbox 
      # so we don't pop up dialog windows while setting used powers
      my $tempcheck = $Whash->{'powercheck'}->get_active;
      $Whash->{'powercheck'}->set_active(FALSE);

      # mark the powers as used
      foreach (@usedpowers) {
	if (/^count: (\d+) (.*)$/) {
	  # restore the count for the power (if present)
	  # note that we subtract one because toggle_power() will
	  # automatically increment it by one
	  $Vars->{'powersbyname'}->{$2}->{'count'} = ($1-1);
	  $_ = $2;
	}
	toggle_power($Whash, $Vars, $Vars->{'powersbyname'}->{$_});
      }

      # restore the "roll power" checkbox
      $Whash->{'powercheck'}->set_active($tempcheck);

      # add the items to the UI
      add_items($Whash, $Vars);

      # mark the items as used
      foreach (@useditems) {
	if (/^count: (\d+) (.*)$/) {
	  # restore the count for the power (if present)
	  # note that we subtract one because toggle_power() will
	  # automatically increment it by one
	  $Vars->{'itemsbyname'}->{$2}->{'count'} = ($1-1);
	  $_ = $2;
	}
	toggle_item($Whash, $Vars, $Vars->{'itemsbyname'}->{$_});
      }



      # toggling powers sets the modified flag, fix it here
      $Vars->{'modified'} = 0;
      update_UI($Whash, $Vars);

      # if defense adjustments were specified, add them and activate them
      # note that this has to be AFTER update_UI() is called
      # as that resets all the adjustments
      if ($ACadj) {
	$Whash->{'defACadjbutton' }->clicked;
	$Whash->{'defACadj'}->set_value($ACadj);
      } else {
	$Whash->{'defACadj'}->set_value($Vars->{'AC'});
      }
      if ($FORTadj) {
	$Whash->{'defFORTadjbutton' }->clicked;
	$Whash->{'defFORTadj'}->set_value($FORTadj);
      } else {
	$Whash->{'defFORTadj'}->set_value($Vars->{'FORT'});
      }

      if ($REFadj) {
	$Whash->{'defREFadjbutton' }->clicked;
	$Whash->{'defREFadj'}->set_value($REFadj);
      } else {
	$Whash->{'defREFadj'}->set_value($Vars->{'REF'});
      }
      if ($WILLadj) {
	$Whash->{'defWILLadjbutton' }->clicked;
	$Whash->{'defWILLadj'}->set_value($WILLadj);
      } else {
	$Whash->{'defWILLadj'}->set_value($Vars->{'WILL'});
      }

    } else {
      print "Can't parse file\n";
    } 

  }
  $dialog->destroy;
}


###########################
#                         #
# write a .dm format file #
#                         #
# format is:              #
# name: value             #
#                         #
# names include           #
#                         #
# type                    #
# version                 #
# file                    #
# currentHP               #
# tempHP                  #
# remaingsurges           #
# condition               #
# actionpoints            #
# magicitemsused          #
# secondwind              #
# usedpower               #
#                         #
###########################
sub write_dm_file {
  my ($Whash, $Vars) = @_;
  my $fh;


  my $f = Gtk2::FileChooserDialog->new("Write PC DM file", 
				       $Whash->{'window'},
				       "save",
				       'gtk-cancel' => 'cancel',
				       'Save' => 'ok');
  my $dmFiles = Gtk2::FileFilter->new;
  $dmFiles->set_name('Dice Minion Files');
  $dmFiles->add_pattern('*.dm');

  my $allFiles = Gtk2::FileFilter->new;
  $allFiles->set_name('All Files');
  $allFiles->add_pattern('*');

  $f->add_filter($dmFiles);
  $f->add_filter($allFiles);

  my $res = $f->run;
  my $file = $f->get_filename();
  $f->destroy;

  if ($res ne 'ok') {
    return;
  }

  my ($name, $path, $suffix) = fileparse($file, qr/\.[^.]*/);
  #print "name = $name, path = $path, suffix = $suffix\n";
  if ($suffix ne ".dm") {
    # add the appropriate suffix to the file if it's not there
    $file .= ".dm";
  }

  # open the file for write
  if (!open ($fh, ">", $file)) {
    # pop up a warning message
    print "Can't open file $file for write!!\n";
    return;
  }

  # write out the fields
  print $fh "ID: DM PC file\n";
  print $fh "Version: $Vars->{'DMversion'}\n";
  print $fh "PCFile: $Vars->{'PCFile'}\n";
  print $fh "curHP: $Vars->{'curHP'}\n";
  print $fh "remainingsurges: $Vars->{'remainingsurges'}\n";
  print $fh "actionpoints: " . $Whash->{'miscadj'}->get_value() . "\n";
  print $fh "magicitemuses: " . $Whash->{'miscadj1'}->get_value() . "\n";

  print $fh "ongoing: " . $Whash->{"hpOngoingEntry"}->get_text . "\n"
    if ($Whash->{"hpOngoingEntry"}->get_text);

  # save modifiers to defenses if they are active
  print $fh "ACadjust: " . $Whash->{'defACadj'}->get_value() . "\n"
    if ($Whash->{'defACadj'}->get_value() != $Vars->{'AC'});
  print $fh "FORTadjust: " . $Whash->{'defFORTadj'}->get_value() . "\n"
    if ($Whash->{'defFORTadj'}->get_value() != $Vars->{'FORT'});
  print $fh "REFadjust: " . $Whash->{'defREFadj'}->get_value() . "\n"
    if ($Whash->{'defREFadj'}->get_value() != $Vars->{'REF'});
  print $fh "WILLadjust: " . $Whash->{'defWILLadj'}->get_value() . "\n"
    if ($Whash->{'defWILLadj'}->get_value() != $Vars->{'WILL'});

  foreach (get_condition_text($Whash)) {
    print $fh "condition: $_\n";
  }

  # write out powers that were used
  my $href = $Vars->{'usedpowers'};
  foreach (keys %$href) {
    if (defined($Vars->{'usedpowers'}->{$_}->{'count'})) {
      # special power, include the count field
      print $fh "usedpower: count: " .
	$Vars->{'usedpowers'}->{$_}->{'count'} . " $_\n";
    } else {
      # binary power, either used or not
      print $fh "usedpower: $_\n";
    }
  }

  # write out items that were used
  $href = $Vars->{'useditems'};
  foreach (keys %$href) {
    # all items include the count field
    print $fh "useditem: count: " .
      $Vars->{'useditems'}->{$_}->{'count'} . " $_\n";
  }

  # close the file
  close $fh;

  $Vars->{'modified'} = 0;

  return;
}


####################################################
#                                                  #
# parse out the Ability, Defense, and Skill fields #
# from the XML structure, $xml_tree, and save them #
# in the $varref fields.                           #
#                                                  #
####################################################
sub xml_parse_ADS {

  my ($varref, $xml_tree) = @_;

  # get the level
  my $xr = $xml_tree->{'CharacterSheet'}->{'Details'};
  $varref->{'name'} = $xr->{'name'};
  $varref->{'level'} = $xr->{'Level'};

  # get rid of extra spaces
  $varref->{'level'} =~ s/\s*([^ ]+)\s*/$1/;

  $xr = $xml_tree->{'CharacterSheet'}->{'StatBlock'}->{'Stat'};
  my ($xr1, $xr2, $xr3);
  my $value;

  # figure out some of the basic stats
  foreach (@$xr) {
    $xr1 = $_;
    foreach (keys %$xr1) {
      $value = $xr1->{'value'};
      $xr2 = $xr1->{'alias'};
      foreach (@$xr2) {
	my $xr3 = $_;
	foreach (keys %$xr3) {
	  # set default variable to the key to simplify cases below
	  $_ = $xr3->{$_};

	  # ability scores
	  $varref->{'Str'}    = $value if (/^str$/);
	  $varref->{'Con'}    = $value if (/^Constitution$/);
	  $varref->{'Dex'}    = $value if (/^Dexterity$/);
	  $varref->{'Int'}    = $value if (/^Intelligence$/);
	  $varref->{'Wis'}    = $value if (/^Wisdom$/);
	  $varref->{'Cha'}    = $value if (/^Charisma$/);
	
	  # ability modifiers
	  $varref->{'StrMod'} = $value if (/^Strength modifier$/);
	  $varref->{'ConMod'} = $value if (/^Constitution modifier$/);
	  $varref->{'DexMod'} = $value if (/^Dexterity modifier$/);
	  $varref->{'IntMod'} = $value if (/^Intelligence modifier$/);
	  $varref->{'WisMod'} = $value if (/^Wisdom modifier$/);
	  $varref->{'ChaMod'} = $value if (/^Charisma modifier$/);

	  # defenses
	  $varref->{'AC'}     = $value if (/^Armor Class$/);
	  $varref->{'FORT'}   = $value if (/^Fortitude$/);
	  $varref->{'REF'}    = $value if (/^Reflex$/);
	  $varref->{'WILL'}   = $value if (/^Will$/);
	  $varref->{'maxHP'}  = $value if (/^Hit Points/);
	  $varref->{'Initiative'}   = $value if (/^Initiative$/);
	  if (/^Healing Surges$/) {
	    # surges are done differently...sigh

	    # first get the basic value
	    $varref->{'surges'} = $value;

#	    # then check if there's a bonus <statadd> statlink="Constitution"
#	    my $ta = $xr1->{'statadd'};
#	    foreach my $statadd (@$ta){
#	      foreach my $statlink (keys %$statadd) {
#		if ($statlink eq "statlink" && 
#		    $statadd->{'statlink'} eq "Constitution") {
#		  # add the constitution modifier to the surges
#		  $varref->{'surgeMod'} = $varref->{'ConMod'};
#		}
#	      }
#	    }
	  }

	  # skills
	  $varref->{'Athletics'}   = $value if (/^Athletics$/);
	  $varref->{'Acrobatics'}   = $value if (/^Acrobatics$/);
	  $varref->{'Arcana'} = $value if (/^Arcana$/);
	  $varref->{'Bluff'}  = $value if (/^Bluff$/);
	  $varref->{'Diplomacy'}  = $value if (/^Diplomacy$/);
	  $varref->{'Dungeoneering'}   = $value if (/^Dungeoneering$/);
	  $varref->{'Endurance'}  = $value if (/^Endurance$/);
	  $varref->{'Heal'}   = $value if (/^Heal$/);
	  $varref->{'History'}   = $value if (/^History$/);
	  $varref->{'Insight'}  = $value if (/^Insight$/);
	  $varref->{'Intimidate'}  = $value if (/^Intimidate$/);
	  $varref->{'Nature'} = $value if (/^Nature$/);
	  $varref->{'Perception'} = $value if (/^Perception$/);
	  $varref->{'Religion'}  = $value if (/^Religion$/);
	  $varref->{'Stealth'}  = $value if (/^Stealth$/);
	  $varref->{'Streetwise'} = $value if (/^Streetwise$/);
	  $varref->{'Thievery'}  = $value if (/^Thievery$/);

	  # misc
	  $varref->{'passp'}  = $value if (/^Passive Perception$/);
	  $varref->{'passi'}  = $value if (/^Passive Insight$/);
	  $varref->{'speed'}  = $value if (/^Speed$/);
	}	# endforeach (my $key keys %$xr3)
      }		# endforeach (@$xr2)  <alias>
    }		# endforeach (keys %$xr1)
  }		# endforeach (@$xr)
}


########################################################
#                                                      #
# parse the powers from the XML structure in $xml_tree #
# and store the results in $varref associative array   #
#                                                      #
# Each power is saved in $varref as a pointer to an    #
# associative array as follows:                        #
#     name,                                            #
#     usage,                                           #
#     type,                                            #
#     weapon = []                                      #
#                                                      #
# where each element in weapon array is a pointer to   #
# an associative array with the following fields       #
#          name                                        #
#          AttackBonus                                 #
#          Conditions                                  #
#          Defense                                     #
#          AttackStat                                  #
#          Damage                                      #
#          Defense                                     #
#                                                      #
#                                                      #
# Note that the add_power() function adds the          #
# following fields to the power entry used when        #
# rendering it:                                        #
#                                                      #
#   label (a Gtk2 label widget)                        #
#   eventbox (a Gtk2 eventbox widget)                  #
#   tooltip (a Gtk2 tooltop widget)                    #
#   fcolor (foreground color for label)                #
#   bcolor (background color for label)                #
#   text (text of label; is name field with padding)   #
#                                                      #
#                                                      #
# E.g.,                                                #
# $varref->{'power'}->[5]->{'weapon'}->[2]->{'Damage'} #
#   which means:                                       #
#   damage from 3rd weapon on 6th power (0-indexed)    #
#                                                      #
########################################################
sub xml_parse_powers {

  my ($varref, $xml_tree) = @_;

  my ($xr, $xr1, $xr2, $xr3);
  my ($value, $powername, $powerusage, $powertype);
  my $weaponarray;

  # create an (anonymous) array to hold the powers
  $varref->{'power'} = [] unless exists $varref->{'power'};


  # fake an entry for second wind
  my $array = $varref->{'power'};
  push @$array, {name => 'Second Wind',
		 usage => 'Encounter',
		 type => 'Standard action',
		 weapon => [
			    { name => 'Unarmed',
			      AttackBonus => 0,
			      Defense => 'Unknown',
			      Damage => 'No'
			    }
			   ]
		};


  # read the powers
  $xr = $xml_tree->{'CharacterSheet'}->{'PowerStats'}->{'Power'};
  foreach (@$xr) {
    # make anonymous array to hold information about weapons 
    $weaponarray = [];

    # figure out if power is at-will/encounter/daily and std/minor/move
    $xr1 = $_;
    $powername = $xr1->{'name'};
    $xr2 = $xr1->{'specific'};
    foreach (@$xr2) {
      $xr3 = $_;
      $powerusage = $xr3->{'content'} 
	if ($xr3->{'name'} eq 'Power Usage');
      $powertype = $xr3->{'content'}
	if ($xr3->{'name'} eq 'Action Type');
    }

    $xr2 = $xr1->{'Weapon'};
    my ($AttackBonus, $Defense, $Damage, $WeaponName);
    foreach (@$xr2) {
      $xr3 = $_;
      $WeaponName = $xr3->{'name'};
      foreach (keys %$xr3) {
	$AttackBonus = $xr3->{$_} if (/^AttackBonus$/);
	$Defense     = $xr3->{$_} if (/^Defense$/);
	$Damage      = $xr3->{$_} if (/^Damage$/);
	# fix weird case when there's no damage but it returns a reference
	# to a hash (that's empty)
	if (ref($Damage) eq "HASH") {
	    $Damage = "No";
	  }
      }

      # strip out leading and ending spaces
      $WeaponName  =~ s/^\s+(.+)\s+$/$1/;
      $AttackBonus =~ s/^\s+(.+)\s+$/$1/;
      $Defense     =~ s/^\s+(.+)\s+$/$1/;
      $Damage      =~ s/^\s+(.+)\s+$/$1/;

      push (@$weaponarray, { name => $WeaponName,
			     AttackBonus => $AttackBonus,
			     Defense => $Defense,
			     Damage => $Damage
			   }
	   );
    }
    my $array = $varref->{'power'};

    # strip out leading and ending spaces
    $powername    =~ s/^\s+(.+)\s+$/$1/;
    $powerusage   =~ s/^\s+(.+)\s+$/$1/;
    $powertype    =~ s/^\s+(.+)\s+$/$1/;

    push (@$array, { name => $powername,
		     usage => $powerusage,
		     type => $powertype,
		     weapon => $weaponarray
		   }
	 );
  }

  # for debugging to list powers we just parsed
  #dump_powers ($varref);

}

###################################
#                                 #
# read in and parse equipment and #
# magic items.  Store as an a     #
#                                 #
#                                 #
###################################
sub xml_parse_items {
  my ($varref, $xml_tree) = @_;
  
  my ($xr, $xr1, $xr2, $xr3);
  my (@Equipped, @Armor, @Weapons, @Magic, @Misc);
  
  # create an (anonymous) array to hold the powers
  $varref->{'power'} = [] unless exists $varref->{'power'};
  
  # read the powers
  $xr = $xml_tree->{'CharacterSheet'}->{'LootTally'}->{'loot'};
  # array of loot tags
  foreach (@$xr) {

    my ($name, $type, $isMagic, $magicName);
    $isMagic = 0;

    if ($_->{'count'} == 0) {
      # skip if we don't actually have it (anymore)
      next;
    }

    $xr1 = $_;
    $xr2 = $xr1->{'RulesElement'};
    # delve into parsing this stuff...
    foreach (@$xr2) {
      if ($_->{'type'} ne "Magic Item") {
	$name = $_->{'name'};
	$type = $_->{'type'};
      } else {
	$magicName = $_->{'name'};
	$isMagic = 1;
      }
    }

    my $equipped = $xr1->{'equip-count'};
    my $aref;

    if ($isMagic) {
      if (defined ($type)) {
	if ($type eq "Armor") {
	  # weapons and armor have compound (magic) names
	  # fix the name
	  $magicName =~ s/($type)/$name/;
	  $name = "(magic)$magicName";
	  if ($equipped) {
	    $aref = \@Equipped;
	  } else {
	    $aref = \@Armor;
	  }
	} elsif ($type eq "Weapon") {
	  # weapons and armor have compound (magic) names
	  # fix the name
	  $magicName =~ s/($type)/$name/;
	  $name = "(magic)$magicName";
	  if ($equipped) {
	    $aref = \@Equipped;
	  } else {
	    $aref = \@Weapons;
	  }
	}
      } else {
	# all other magic things only have their magic name
	$name = "(magic)$magicName";
	if ($equipped) {
	  $aref = \@Equipped;
	} else {
	  $aref = \@Magic;
	}
      }
    } else {
      # everything else just has its name
      if ($equipped) {
	$aref = \@Equipped;
      } else {
	$aref = \@Misc;
      }
    }
    push @$aref, {name => $name};

  }

  $varref->{'items'} = { Equipped => \@Equipped,
			 Armor    => \@Armor,
			 Weapons  => \@Weapons,
			 Magic    => \@Magic,
			 Misc     => \@Misc,
		       };
}


#
#
##################################################
# Read in and parse the XML character sheet file #
#                                                #
# Input is: XML file name, XML reference for     #
# parsed input, and a variable reference to      #
# store PC's data.                               #
#                                                #
##################################################
#
#
sub read_xml {

  my ($file, $xmlref, $varref) = @_;

  # Note: we need to use "forcearray" because for parsing consistency 
  # we always want those elements to be an array, even if there is only
  # one of them
  my $xs = XML::Simple->new();
  eval {
    $xmlref = $xs->XMLin($file, 
			 forcearray => ['alias', 'Weapon', 'RulesElement'], 
			 KeyAttr => {});
  };
  if ($@) {
    print "XML read error!\n";
    return 1;
  }

  # get the level
  my $xr = $xmlref->{'CharacterSheet'}->{'Details'};
  $varref->{'name'} = $xr->{'name'};
  $varref->{'level'} = $xr->{'Level'};

  # get rid of extra spaces
  $varref->{'level'} =~ m/(\d+)/;
  $varref->{'level'} = $1;

  # parse the Ability, Defenese, and Skill fields
  xml_parse_ADS ($varref, $xmlref);

  # parase the powers
  xml_parse_powers ($varref, $xmlref);

  # parse the items
  xml_parse_items ($varref, $xmlref);

  $xr = $xmlref->{'CharacterSheet'}->{'StatBlock'}->{'Stat'};
  my ($xr1, $xr2, $xr3);
  my $value;

  # read the race and class
  $xr = $xmlref->{'CharacterSheet'}->{'RulesElementTally'}->{'RulesElement'};
  foreach (@$xr) {
    $xr1 = $_;
    my ($name, $type) = "";
    foreach (keys %$xr1) {
      $name = $xr1->{$_} if (/^name$/);
      $type = $xr1->{$_} if (/^type$/);
      $varref->{'race'} = $name if  (defined ($type) && $type eq 'Race');
      $varref->{'class'} = $name if (defined ($type) && $type eq 'Class'); 

    }
  }

  # compute bloodied and healing surge values
  fix_stats($varref);

#  foreach (keys %$varref) {
#    print "$_ : [" . $varref->{$_} . "]\n";
#  }
  return 0;
}


#
#
######################
# Fix up some stats  #
######################
#
#
sub fix_stats {
  my $varref = shift;

  # compute the bloodied and healing surge values
  $varref->{'bloodied'}  = int($varref->{'maxHP'}/2);
  $varref->{'surgeval'}  = int($varref->{'maxHP'}/4);

  # If there was a surgeMod detected, then add it to the surge value
  $varref->{'surgeval'} += $varref->{'ConMod'}
    if ($varref->{'race'} eq 'Dragonborn');

  # set PC action points to 1
  $varref->{'defaultactionpoints'} = 1;
}

#
# debugging routine to dump out the $var->{'power'} 
# data structure to stdout
#
sub dump_powers {

  my $ref = shift;

  my $powers = $ref->{'power'};
  foreach (@$powers) {
    # print the header information
    print "Name: $_->{'name'} ($_->{'usage'},  $_->{'type'})\n";
    my $weapons = $_->{'weapon'};
    # iterate through every weapon and print their stats
    foreach (@$weapons) {
      print "    $_->{'name'}: $_->{'AttackBonus'} Vs. $_->{'Defense'}, $_->{'Damage'} damage\n";
    }
  }
}

1;
